@@ -1,10 +1,11 @@
 # pylint: disable=wildcard-import
 # pylint: disable=wrong-import-position
 
-__version__ = "9.2.21"
+__version__ = "9.2.39"
 
 if bytes is str:
-    raise Exception("""
+    raise Exception(
+        """
 
 =-=-=-=-=-=-=-=-=-=-=-=-=  WELCOME TO THE FUTURE!  =-=-=-=-=-=-=-=-=-=-=-=-=-=
 
@@ -17,16 +18,20 @@ vaccinations. It hurts a little bit initially but in the end it's worth it.
 For more information, see here: https://docs.angr.io/appendix/migration
 
 Good luck!
-""")
+"""
+    )
 
 from .utils.formatting import setup_terminal
+
 setup_terminal()
 del setup_terminal
 
 # let's set up some bootstrap logging
 import logging
+
 logging.getLogger("angr").addHandler(logging.NullHandler())
 from .misc.loggers import Loggers
+
 loggers = Loggers()
 del Loggers
 del logging
@@ -34,6 +39,7 @@ del logging
 # import hook: pkg_resources is too slow to import, but it is used by some other packages that angr depends on. while
 # we upstream our fixes, we replace it with a light-weight solution.
 from .misc.import_hooks import import_fake_pkg_resources
+
 import_fake_pkg_resources(force=False)
 
 # this must happen first, prior to initializing analyses
@@ -41,6 +47,7 @@ from .sim_procedure import SimProcedure
 from .procedures import SIM_PROCEDURES, SimProcedures, SIM_LIBRARIES
 
 from . import sim_options
+
 options = sim_options  # alias
 
 # enums
@@ -61,14 +68,21 @@ from . import analyses
 from . import knowledge_plugins
 from . import exploration_techniques
 from .exploration_techniques import ExplorationTechnique
-from . import type_backend
 from . import sim_type as types
 from .state_hierarchy import StateHierarchy
 
 from .sim_state import SimState
 from . import engines
 from .calling_conventions import DEFAULT_CC, SYSCALL_CC, PointerWrapper, SimCC
-from .storage.file import SimFileBase, SimFile, SimPackets, SimFileStream, SimPacketsStream, SimFileDescriptor, SimFileDescriptorDuplex
+from .storage.file import (
+    SimFileBase,
+    SimFile,
+    SimPackets,
+    SimFileStream,
+    SimPacketsStream,
+    SimFileDescriptor,
+    SimFileDescriptorDuplex,
+)
 from .state_plugins.filesystem import SimMount, SimHostFilesystem
 from .state_plugins.heap import SimHeapBrk, SimHeapPTMalloc, PTChunk
 from . import concretization_strategies
