@@ -1,9 +1,10 @@
+from typing import List, Set, Optional
 
 import networkx
 
 
 class SCCPlaceholder:
-    __slots__ = ['scc_id']
+    __slots__ = ["scc_id"]
 
     def __init__(self, scc_id):
         self.scc_id = scc_id
@@ -12,7 +13,7 @@ class SCCPlaceholder:
         return isinstance(other, SCCPlaceholder) and other.scc_id == self.scc_id
 
     def __hash__(self):
-        return hash('scc_placeholder_%d' % self.scc_id)
+        return hash("scc_placeholder_%d" % self.scc_id)
 
 
 class CFGUtils:
@@ -77,7 +78,7 @@ class CFGUtils:
             else:
                 for n in scc:
                     predecessors = graph.predecessors(n)
-                    if any([ p not in scc for p in predecessors]):
+                    if any([p not in scc for p in predecessors]):
                         widening_addrs.add(n.addr)
                         break
 
@@ -103,7 +104,9 @@ class CFGUtils:
         return sorted(nodes, key=lambda n: addrs_to_index[n.addr], reverse=True)
 
     @staticmethod
-    def quasi_topological_sort_nodes(graph, nodes=None):
+    def quasi_topological_sort_nodes(
+        graph: networkx.DiGraph, nodes: Optional[List] = None, loop_heads: Optional[List] = None
+    ) -> List:
         """
         Sort a given set of nodes from a graph based on the following rules:
 
@@ -113,23 +116,23 @@ class CFGUtils:
         Following the above rules gives us a quasi-topological sorting of nodes in the graph. It also works for cyclic
         graphs.
 
-        :param networkx.DiGraph graph: A local transition graph of the function.
-        :param iterable nodes: A list of nodes to sort. None if you want to sort all nodes inside the graph.
-        :return: A list of ordered nodes.
-        :rtype: list
+        :param graph:       A local transition graph of the function.
+        :param nodes:       A list of nodes to sort. None if you want to sort all nodes inside the graph.
+        :param loop_heads:  A list of nodes that should be treated loop heads.
+        :return:            A list of ordered nodes.
         """
 
         # fast path for single node graphs
         if graph.number_of_nodes() == 1:
             if nodes is None:
                 return list(graph.nodes)
-            return [ n for n in graph.nodes() if n in nodes ]
+            return [n for n in graph.nodes() if n in nodes]
 
         # make a copy to the graph since we are gonna modify it
         graph_copy = networkx.DiGraph()
 
         # find all strongly connected components in the graph
-        sccs = [ scc for scc in networkx.strongly_connected_components(graph) if len(scc) > 1 ]
+        sccs = [scc for scc in networkx.strongly_connected_components(graph) if len(scc) > 1]
 
         # collapse all strongly connected components
         for src, dst in graph.edges():
@@ -141,8 +144,12 @@ class CFGUtils:
                 dst = SCCPlaceholder(scc_index)
 
             if isinstance(src, SCCPlaceholder) and isinstance(dst, SCCPlaceholder) and src == dst:
+                if src not in graph_copy:
+                    graph_copy.add_node(src)
                 continue
             if src == dst:
+                if src not in graph_copy:
+                    graph_copy.add_node(src)
                 continue
 
             graph_copy.add_edge(src, dst)
@@ -156,10 +163,10 @@ class CFGUtils:
         # topological sort on acyclic graph `graph_copy`
         tmp_nodes = networkx.topological_sort(graph_copy)
 
-        ordered_nodes = [ ]
+        ordered_nodes = []
         for n in tmp_nodes:
             if isinstance(n, SCCPlaceholder):
-                CFGUtils._append_scc(graph, ordered_nodes, sccs[n.scc_id])
+                CFGUtils._append_scc(graph, ordered_nodes, sccs[n.scc_id], loop_heads=loop_heads)
             else:
                 ordered_nodes.append(n)
 
@@ -167,45 +174,53 @@ class CFGUtils:
             return ordered_nodes
 
         nodes = set(nodes)
-        ordered_nodes = [ n for n in ordered_nodes if n in nodes ]
+        ordered_nodes = [n for n in ordered_nodes if n in nodes]
         return ordered_nodes
 
     @staticmethod
     def _components_index_node(components, node):
-
         for i, comp in enumerate(components):
             if node in comp:
                 return i
         return None
 
     @staticmethod
-    def _append_scc(graph, ordered_nodes, scc):
+    def _append_scc(graph: networkx.DiGraph, ordered_nodes: List, scc: Set, loop_heads: Optional[List] = None) -> None:
         """
         Append all nodes from a strongly connected component to a list of ordered nodes and ensure the topological
         order.
 
-        :param networkx.DiGraph graph: The graph where all nodes belong to.
-        :param list ordered_nodes:     Ordered nodes.
-        :param iterable scc:           A set of nodes that forms a strongly connected component in the graph.
-        :return:                       None
+        :param graph: The graph where all nodes belong to.
+        :param ordered_nodes:     Ordered nodes.
+        :param scc:           A set of nodes that forms a strongly connected component in the graph.
         """
 
-        # find the first node in the strongly connected component that is the successor to any node in ordered_nodes
         loop_head = None
-        for parent_node in reversed(ordered_nodes):
+
+        if loop_heads is not None:
+            # find the first node that appears in loop_heads
             for n in scc:
-                if n in graph[parent_node]:
+                if n in loop_heads:
                     loop_head = n
                     break
 
-            if loop_head is not None:
-                break
+        if loop_head is None:
+            # find the first node in the strongly connected component that is the successor to any node in
+            # ordered_nodes
+            for parent_node in reversed(ordered_nodes):
+                for n in scc:
+                    if n in graph[parent_node]:
+                        loop_head = n
+                        break
+
+                if loop_head is not None:
+                    break
 
         if loop_head is None:
             # randomly pick one
             loop_head = next(iter(scc))
 
-        subgraph = graph.subgraph(scc).copy()  # type: networkx.DiGraph
+        subgraph: networkx.DiGraph = graph.subgraph(scc).copy()
         for src, _ in list(subgraph.in_edges(loop_head)):
             subgraph.remove_edge(src, loop_head)
 
