@@ -7,10 +7,9 @@ class ConvShlShr(PeepholeOptimizationExprBase):
     __slots__ = ()
 
     NAME = "(expr << P) >> Q => (expr & mask) >> R"
-    expr_classes = (BinaryOp, )  # all expressions are allowed
+    expr_classes = (BinaryOp,)  # all expressions are allowed
 
     def optimize(self, expr: BinaryOp):
-
         # (Conv(M->N, expr) << P) >> Q  ==>  (Conv(M->N, expr) & bitmask) >> (Q-P), where
         #       Q >= P, and
         #       M < N, and
@@ -28,7 +27,7 @@ class ConvShlShr(PeepholeOptimizationExprBase):
                         bitmask = (1 << (n - p)) - 1
                         and_expr = BinaryOp(
                             None,
-                            'And',
+                            "And",
                             (
                                 Convert(expr_a.idx, m, n, False, expr_a.operand, **expr_a.tags),
                                 Const(None, None, bitmask, n),
@@ -40,7 +39,7 @@ class ConvShlShr(PeepholeOptimizationExprBase):
                         )
                         return BinaryOp(
                             None,
-                            'Shr',
+                            "Shr",
                             (
                                 and_expr,
                                 Const(None, None, q - p, and_expr.bits),
