@@ -8,7 +8,13 @@ from ailment.statement import Statement, Assignment, Call
 
 from ..ailblock_walker import AILBlockWalker
 from ..sequence_walker import SequenceWalker
-from ..structurer_nodes import ConditionNode, ConditionalBreakNode, LoopNode, CascadingConditionNode
+from ..structuring.structurer_nodes import (
+    ConditionNode,
+    ConditionalBreakNode,
+    LoopNode,
+    CascadingConditionNode,
+    SwitchCaseNode,
+)
 
 if TYPE_CHECKING:
     from angr.sim_variable import SimVariable
@@ -16,13 +22,15 @@ if TYPE_CHECKING:
 
 
 class LocationBase:
-
     __slots__ = ()
 
 
 class StatementLocation(LocationBase):
-
-    __slots__ = ('block_addr', 'block_idx', 'stmt_idx', )
+    __slots__ = (
+        "block_addr",
+        "block_idx",
+        "stmt_idx",
+    )
 
     def __init__(self, block_addr, block_idx, stmt_idx):
         self.block_addr = block_addr
@@ -36,18 +44,24 @@ class StatementLocation(LocationBase):
         return hash((StatementLocation, self.block_addr, self.block_idx, self.stmt_idx))
 
     def __eq__(self, other):
-        return isinstance(other, StatementLocation) and \
-            self.block_addr == other.block_addr and \
-            self.block_idx == other.block_idx and \
-            self.stmt_idx == other.stmt_idx
+        return (
+            isinstance(other, StatementLocation)
+            and self.block_addr == other.block_addr
+            and self.block_idx == other.block_idx
+            and self.stmt_idx == other.stmt_idx
+        )
 
     def copy(self):
         return StatementLocation(self.block_addr, self.block_idx, self.stmt_idx)
 
 
 class ExpressionLocation(LocationBase):
-
-    __slots__ = ('block_addr', 'block_idx', 'stmt_idx', 'expr_idx', )
+    __slots__ = (
+        "block_addr",
+        "block_idx",
+        "stmt_idx",
+        "expr_idx",
+    )
 
     def __init__(self, block_addr, block_idx, stmt_idx, expr_idx):
         self.block_addr = block_addr
@@ -65,18 +79,22 @@ class ExpressionLocation(LocationBase):
         return hash((ExpressionLocation, self.block_addr, self.block_idx, self.stmt_idx, self.expr_idx))
 
     def __eq__(self, other):
-        return isinstance(other, ExpressionLocation) and \
-                self.block_addr == other.block_addr and \
-                self.block_idx == other.block_idx and \
-                self.stmt_idx == other.stmt_idx and \
-                self.expr_idx == other.expr_idx
+        return (
+            isinstance(other, ExpressionLocation)
+            and self.block_addr == other.block_addr
+            and self.block_idx == other.block_idx
+            and self.stmt_idx == other.stmt_idx
+            and self.expr_idx == other.expr_idx
+        )
 
 
 class ConditionLocation(LocationBase):
+    __slots__ = (
+        "node_addr",
+        "case_idx",
+    )
 
-    __slots__ = ('node_addr', 'case_idx', )
-
-    def __init__(self, cond_node_addr, case_idx: Optional[int]=None):
+    def __init__(self, cond_node_addr, case_idx: Optional[int] = None):
         self.node_addr = cond_node_addr
         self.case_idx = case_idx
 
@@ -87,14 +105,15 @@ class ConditionLocation(LocationBase):
         return hash((ConditionLocation, self.node_addr, self.case_idx))
 
     def __eq__(self, other):
-        return isinstance(other, ConditionLocation) and \
-                self.node_addr == other.node_addr and \
-                self.case_idx == other.case_idx
+        return (
+            isinstance(other, ConditionLocation)
+            and self.node_addr == other.node_addr
+            and self.case_idx == other.case_idx
+        )
 
 
 class ConditionalBreakLocation(LocationBase):
-
-    __slots__ = ('node_addr', )
+    __slots__ = ("node_addr",)
 
     def __init__(self, node_addr):
         self.node_addr = node_addr
@@ -106,8 +125,7 @@ class ConditionalBreakLocation(LocationBase):
         return hash((ConditionalBreakLocation, self.node_addr))
 
     def __eq__(self, other):
-        return isinstance(other, ConditionalBreakLocation) and \
-                self.node_addr == other.node_addr
+        return isinstance(other, ConditionalBreakLocation) and self.node_addr == other.node_addr
 
 
 class ExpressionUseFinder(AILBlockWalker):
@@ -131,15 +149,19 @@ class ExpressionUseFinder(AILBlockWalker):
     the second statement.
     """
 
-    __slots__ = ('uses', 'has_load', )
+    __slots__ = (
+        "uses",
+        "has_load",
+    )
 
     def __init__(self):
         super().__init__()
         self.uses = defaultdict(set)
         self.has_load = False
 
-    def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement],
-                     block: Optional[Block]) -> Any:
+    def _handle_expr(
+        self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement], block: Optional[Block]
+    ) -> Any:
         if isinstance(expr, ailment.Register) and expr.variable is not None:
             if not (isinstance(stmt, ailment.Stmt.Assignment) and stmt.dst is expr):
                 if block is not None:
@@ -149,8 +171,9 @@ class ExpressionUseFinder(AILBlockWalker):
             return None
         return super()._handle_expr(expr_idx, expr, stmt_idx, stmt, block)
 
-    def _handle_Load(self, expr_idx: int, expr: ailment.Expr.Load, stmt_idx: int, stmt: Statement,
-                     block: Optional[Block]):
+    def _handle_Load(
+        self, expr_idx: int, expr: ailment.Expr.Load, stmt_idx: int, stmt: Statement, block: Optional[Block]
+    ):
         self.has_load = True
         return super()._handle_Load(expr_idx, expr, stmt_idx, stmt, block)
 
@@ -159,25 +182,27 @@ class ExpressionCounter(SequenceWalker):
     """
     Find all expressions that are assigned once and only used once.
     """
+
     def __init__(self, node, variable_manager):
         handlers = {
             ConditionalBreakNode: self._handle_ConditionalBreak,
             ConditionNode: self._handle_Condition,
             LoopNode: self._handle_Loop,
+            SwitchCaseNode: self._handle_SwitchCase,
             ailment.Block: self._handle_Block,
         }
 
         # each element in the set is a tuple of (source of the assignment statement, a tuple of unified variables that
         # the current assignment depends on, StatementLocation of the assignment statement, a Boolean variable that
         # indicates if ExpressionUseFinder has succeeded or not)
-        self.assignments: DefaultDict[Any,Set[Tuple]] = defaultdict(set)
-        self.uses = { }
-        self._variable_manager: 'VariableManagerInternal' = variable_manager
+        self.assignments: DefaultDict[Any, Set[Tuple]] = defaultdict(set)
+        self.uses = {}
+        self._variable_manager: "VariableManagerInternal" = variable_manager
 
         super().__init__(handlers)
         self.walk(node)
 
-    def _u(self, v) -> Optional['SimVariable']:
+    def _u(self, v) -> Optional["SimVariable"]:
         """
         Get unified variable for a given variable.
         """
@@ -194,23 +219,28 @@ class ExpressionCounter(SequenceWalker):
                         # dependency
                         dependency_finder = ExpressionUseFinder()
                         dependency_finder.walk_expression(stmt.src)
-                        dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
-                        self.assignments[u].add((stmt.src,
-                                                 dependencies,
-                                                 StatementLocation(node.addr, node.idx, idx),
-                                                 dependency_finder.has_load))
-            if (isinstance(stmt, ailment.Stmt.Call)
-                    and isinstance(stmt.ret_expr, ailment.Expr.Register)
-                    and stmt.ret_expr.variable is not None):
+                        dependencies = tuple({self._u(v) for v in dependency_finder.uses})
+                        self.assignments[u].add(
+                            (
+                                stmt.src,
+                                dependencies,
+                                StatementLocation(node.addr, node.idx, idx),
+                                dependency_finder.has_load,
+                            )
+                        )
+            if (
+                isinstance(stmt, ailment.Stmt.Call)
+                and isinstance(stmt.ret_expr, ailment.Expr.Register)
+                and stmt.ret_expr.variable is not None
+            ):
                 u = self._u(stmt.ret_expr.variable)
                 if u is not None:
                     dependency_finder = ExpressionUseFinder()
                     dependency_finder.walk_expression(stmt)
-                    dependencies = tuple(set(self._u(v) for v in dependency_finder.uses))
-                    self.assignments[u].add((stmt,
-                                             dependencies,
-                                             StatementLocation(node.addr, node.idx, idx),
-                                             dependency_finder.has_load))
+                    dependencies = tuple({self._u(v) for v in dependency_finder.uses})
+                    self.assignments[u].add(
+                        (stmt, dependencies, StatementLocation(node.addr, node.idx, idx), dependency_finder.has_load)
+                    )
 
         # walk the block and find uses of variables
         use_finder = ExpressionUseFinder()
@@ -261,15 +291,19 @@ class ExpressionCounter(SequenceWalker):
             self._collect_uses(node.condition, ConditionLocation(node.addr))
         return super()._handle_Loop(node, **kwargs)
 
+    def _handle_SwitchCase(self, node: SwitchCaseNode, **kwargs):
+        self._collect_uses(node.switch_expr, ConditionLocation(node.addr))
+        return super()._handle_SwitchCase(node, **kwargs)
+
 
 class ExpressionReplacer(AILBlockWalker):
     def __init__(self, assignments: Dict, uses: Dict, variable_manager):
         super().__init__()
         self._assignments = assignments
         self._uses = uses
-        self._variable_manager: 'VariableManagerInternal' = variable_manager
+        self._variable_manager: "VariableManagerInternal" = variable_manager
 
-    def _u(self, v) -> Optional['SimVariable']:
+    def _u(self, v) -> Optional["SimVariable"]:
         """
         Get unified variable for a given variable.
         """
@@ -296,8 +330,9 @@ class ExpressionReplacer(AILBlockWalker):
             new_stmt = Assignment(stmt.idx, dst, src, **stmt.tags)
             block.statements[stmt_idx] = new_stmt
 
-    def _handle_expr(self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement],
-                     block: Optional[Block]) -> Any:
+    def _handle_expr(
+        self, expr_idx: int, expr: Expression, stmt_idx: int, stmt: Optional[Statement], block: Optional[Block]
+    ) -> Any:
         if isinstance(expr, ailment.Register) and expr.variable is not None:
             unified_var = self._u(expr.variable)
             if unified_var in self._uses:
@@ -308,11 +343,11 @@ class ExpressionReplacer(AILBlockWalker):
 
 class ExpressionFolder(SequenceWalker):
     def __init__(self, assignments: Dict, uses: Dict, node, variable_manager):
-
         handlers = {
             ailment.Block: self._handle_Block,
             ConditionNode: self._handle_Condition,
             ConditionalBreakNode: self._handle_ConditionalBreak,
+            SwitchCaseNode: self._handle_SwitchCase,
         }
 
         super().__init__(handlers)
@@ -321,7 +356,7 @@ class ExpressionFolder(SequenceWalker):
         self._variable_manager = variable_manager
         self.walk(node)
 
-    def _u(self, v) -> Optional['SimVariable']:
+    def _u(self, v) -> Optional["SimVariable"]:
         """
         Get unified variable for a given variable.
         """
@@ -329,16 +364,18 @@ class ExpressionFolder(SequenceWalker):
 
     def _handle_Block(self, node: ailment.Block, **kwargs):
         # Walk the block to remove each assignment and replace uses of each variable
-        new_stmts = [ ]
+        new_stmts = []
         for stmt in node.statements:
             if isinstance(stmt, ailment.Stmt.Assignment):
                 if isinstance(stmt.dst, ailment.Expr.Register) and stmt.dst.variable is not None:
                     if stmt.dst.variable in self._assignments:
                         # remove this statement
                         continue
-            if (isinstance(stmt, ailment.Stmt.Call)
-                    and isinstance(stmt.ret_expr, ailment.Expr.Register)
-                    and stmt.ret_expr.variable is not None):
+            if (
+                isinstance(stmt, ailment.Stmt.Call)
+                and isinstance(stmt.ret_expr, ailment.Expr.Register)
+                and stmt.ret_expr.variable is not None
+            ):
                 unified_var = self._u(stmt.ret_expr.variable)
                 if unified_var in self._assignments:
                     # remove this statement
@@ -396,12 +433,24 @@ class ExpressionFolder(SequenceWalker):
 
         return super()._handle_Loop(node, **kwargs)
 
+    def _handle_SwitchCase(self, node: SwitchCaseNode, **kwargs):
+        replacer = ExpressionReplacer(self._assignments, self._uses, self._variable_manager)
+
+        r = replacer.walk_expression(node.switch_expr)
+        if r is not None and r is not node.switch_expr:
+            node.switch_expr = r
+
+        return super()._handle_SwitchCase(node, **kwargs)
+
 
 class StoreStatementFinder(SequenceWalker):
     """
     Determine if there are any Store statements between two given statements.
+
+    This class overrides _handle_Sequence() and _handle_MultiNode() to ensure they traverse nodes from top to bottom.
     """
-    def __init__(self, node, intervals: Iterable[Tuple[StatementLocation,LocationBase]]):
+
+    def __init__(self, node, intervals: Iterable[Tuple[StatementLocation, LocationBase]]):
         handlers = {
             ConditionNode: self._handle_Condition,
             CascadingConditionNode: self._handle_CascadingCondition,
@@ -411,9 +460,9 @@ class StoreStatementFinder(SequenceWalker):
 
         self._intervals = intervals
 
-        self._start_to_ends: DefaultDict[StatementLocation,Set[LocationBase]] = defaultdict(set)
-        self._end_to_starts: DefaultDict[LocationBase,Set[StatementLocation]] = defaultdict(set)
-        self.interval_to_hasstore: Dict[Tuple[StatementLocation,StatementLocation],bool] = { }
+        self._start_to_ends: DefaultDict[StatementLocation, Set[LocationBase]] = defaultdict(set)
+        self._end_to_starts: DefaultDict[LocationBase, Set[StatementLocation]] = defaultdict(set)
+        self.interval_to_hasstore: Dict[Tuple[StatementLocation, StatementLocation], bool] = {}
         for start, end in intervals:
             self._start_to_ends[start].add(end)
             self._end_to_starts[end].add(start)
@@ -423,6 +472,20 @@ class StoreStatementFinder(SequenceWalker):
         super().__init__(handlers)
         self.walk(node)
 
+    def _handle_Sequence(self, node, **kwargs):
+        i = 0
+        while i < len(node.nodes):
+            node_ = node.nodes[i]
+            self._handle(node_, parent=node, index=i)
+            i += 1
+
+    def _handle_MultiNode(self, node, **kwargs):
+        i = 0
+        while i < len(node.nodes):
+            node_ = node.nodes[i]
+            self._handle(node_, parent=node, index=i)
+            i += 1
+
     def _handle_Block(self, node: ailment.Block, **kwargs):
         stmt_loc = StatementLocation(node.addr, node.idx, None)
         for idx, stmt in enumerate(node.statements):
