@@ -1,4 +1,3 @@
-
 from collections import defaultdict
 
 from cle.loader import MetaELF
@@ -47,14 +46,16 @@ class SimEngineInitFinderVEX(
             if not obj.has_memory:
                 # Objects without memory are definitely uninitialized
                 return True
-            section = obj.find_section_containing(addr)  # type: Section
+            section: Section = obj.find_section_containing(addr)
             if section is not None:
-                return section.name in {'.bss', }
+                return section.name in {
+                    ".bss",
+                }
 
             if isinstance(obj, MetaELF):
                 # for ELFs, if p_memsz >= p_filesz, the extra bytes are considered NOBITS
                 # https://docs.oracle.com/cd/E19120-01/open.solaris/819-0690/gjpww/index.html
-                segment = obj.find_segment_containing(addr)  # type: Segment
+                segment: Segment = obj.find_segment_containing(addr)
                 if segment is not None and segment.memsize > segment.filesize:
                     return segment.vaddr + segment.filesize <= addr < segment.vaddr + segment.memsize
         return False
@@ -99,9 +100,7 @@ class SimEngineInitFinderVEX(
                                 data_size = self.tyenv.sizeof(stmt.data.tmp)
                                 data_v = claripy.BVV(data_v, data_size)
                             if not self.pointers_only or self._is_pointer(data_v):
-                                self.overlay.store(addr_v, data_v,
-                                                   endness=self.project.arch.memory_endness
-                                                   )
+                                self.overlay.store(addr_v, data_v, endness=self.project.arch.memory_endness)
 
     def _handle_StoreG(self, stmt):
         blockloc = self._codeloc(block_only=True)
@@ -134,9 +133,7 @@ class SimEngineInitFinderVEX(
                 data_v = claripy.BVV(data_v, data_size)
 
             if not self.pointers_only or self._is_pointer(data_v):
-                self.overlay.store(addr_v, data_v,
-                                   endness=self.project.arch.memory_endness
-                                   )
+                self.overlay.store(addr_v, data_v, endness=self.project.arch.memory_endness)
 
     #
     # Expression handlers
@@ -166,7 +163,16 @@ class InitializationFinder(ForwardAnalysis, Analysis):  # pylint:disable=abstrac
     on.
     """
 
-    def __init__(self, func=None, func_graph=None, block=None, max_iterations=1, replacements=None, overlay=None, pointers_only=False):
+    def __init__(
+        self,
+        func=None,
+        func_graph=None,
+        block=None,
+        max_iterations=1,
+        replacements=None,
+        overlay=None,
+        pointers_only=False,
+    ):
         self.pointers_only = pointers_only
         if func is not None:
             if block is not None:
@@ -174,21 +180,24 @@ class InitializationFinder(ForwardAnalysis, Analysis):  # pylint:disable=abstrac
             # traversing a function
             graph_visitor = FunctionGraphVisitor(func, func_graph)
             if replacements is None:
-                prop = self.project.analyses[PropagatorAnalysis].prep()(func=func, func_graph=func_graph,
-                                                        base_state=self.project.factory.blank_state())
+                prop = self.project.analyses[PropagatorAnalysis].prep()(
+                    func=func, func_graph=func_graph, base_state=self.project.factory.blank_state()
+                )
                 replacements = prop.replacements
         elif block is not None:
             # traversing a block
             graph_visitor = SingleNodeGraphVisitor(block)
             if replacements is None:
-                prop = self.project.analyses[PropagatorAnalysis].prep()(block=block,
-                                                        base_state=self.project.factory.blank_state())
+                prop = self.project.analyses[PropagatorAnalysis].prep()(
+                    block=block, base_state=self.project.factory.blank_state()
+                )
                 replacements = prop.replacements
         else:
-            raise ValueError('Unsupported analysis target.')
+            raise ValueError("Unsupported analysis target.")
 
-        ForwardAnalysis.__init__(self, order_jobs=True, allow_merging=True, allow_widening=False,
-                                 graph_visitor=graph_visitor)
+        ForwardAnalysis.__init__(
+            self, order_jobs=True, allow_merging=True, allow_widening=False, graph_visitor=graph_visitor
+        )
 
         self._function = func
         self._max_iterations = max_iterations
@@ -203,7 +212,9 @@ class InitializationFinder(ForwardAnalysis, Analysis):  # pylint:disable=abstrac
             self.overlay_state = self.project.factory.blank_state()
             self.overlay = self.overlay_state.memory
 
-        self._engine_vex = SimEngineInitFinderVEX(self.project, replacements, self.overlay, pointers_only=self.pointers_only)
+        self._engine_vex = SimEngineInitFinderVEX(
+            self.project, replacements, self.overlay, pointers_only=self.pointers_only
+        )
         self._engine_ail = None
 
         self._analyze()
@@ -225,7 +236,6 @@ class InitializationFinder(ForwardAnalysis, Analysis):  # pylint:disable=abstrac
         return None
 
     def _run_on_node(self, node, state):
-
         block = self.project.factory.block(node.addr, node.size, opt_level=1, cross_insn_opt=False)
         block_key = node.addr
         engine = self._engine_vex
