@@ -1,4 +1,3 @@
-
 import logging
 
 from ..block import SootBlockNode
@@ -12,28 +11,26 @@ l = logging.getLogger(name=__name__)
 
 
 class VariableTypes:
-    Iterator = 'Iterator'
-    HasNext = 'HasNext'
-    Next = 'Next'
+    Iterator = "Iterator"
+    HasNext = "HasNext"
+    Next = "Next"
 
 
 class AnnotatedVariable:
-
-    __slots__ = ['variable', 'type']
+    __slots__ = ["variable", "type"]
 
     def __init__(self, variable, type_):
         self.variable = variable
         self.type = type_
 
     def __repr__(self):
-        return "%s//%s" % (self.variable, self.type)
+        return f"{self.variable}//{self.type}"
 
     def __eq__(self, other):
         return type(other) is AnnotatedVariable and other.type == self.type and other.variable == self.variable
 
 
 class Condition:
-
     Equal = "=="
     NotEqual = "!="
 
@@ -43,16 +40,15 @@ class Condition:
         self.val1 = val1
 
     def __repr__(self):
-        return "%s %s %s" % (self.val0, self.op, self.val1)
+        return f"{self.val0} {self.op} {self.val1}"
 
     @classmethod
     def from_opstr(cls, opstr):
-
         mapping = {
-            'eq': cls.Equal,
-            '==': cls.Equal,
-            'ne': cls.NotEqual,
-            '!=': cls.NotEqual,
+            "eq": cls.Equal,
+            "==": cls.Equal,
+            "ne": cls.NotEqual,
+            "!=": cls.NotEqual,
         }
 
         return mapping.get(opstr, None)
@@ -66,7 +62,6 @@ class SootBlockProcessor:
         self.defuse = defuse
 
     def process(self):
-
         if not isinstance(self.block, SootBlockNode):
             raise AngrLoopAnalysisError("Got an unexpected type of block %s." % type(self.block))
 
@@ -74,7 +69,6 @@ class SootBlockProcessor:
             return None
 
         for stmt in self.block.stmts:
-
             func_name = "_handle_%s" % (stmt.__class__.__name__)
 
             if hasattr(self, func_name):
@@ -98,7 +92,6 @@ class SootBlockProcessor:
         return False
 
     def _expr(self, expr):
-
         func_name = "_handle_%s" % (expr.__class__.__name__)
 
         if hasattr(self, func_name):
@@ -111,26 +104,29 @@ class SootBlockProcessor:
     #
 
     def _handle_AssignStmt(self, stmt):
-
         left_op, right_op = stmt.left_op, stmt.right_op
 
         expr = self._expr(right_op)
         if expr is not None:
+            try:
+                from pysoot.sootir.soot_value import SootLocal
+            except ImportError:
+                l.error("Please install PySoot before analyzing Java byte code.")
+                raise
             if isinstance(left_op, SootLocal):
                 # Log a def for the local variable
                 self.state.locals[left_op.name] = expr
 
     def _handle_IfStmt(self, stmt):
-
         target = stmt.target
 
         # is it jumping outside the loop?
         if not self._stmt_inside_loop(target):
-
-            cond = Condition(Condition.from_opstr(stmt.condition.op),
-                             self._expr(stmt.condition.value1),
-                             self._expr(stmt.condition.value2),
-                             )
+            cond = Condition(
+                Condition.from_opstr(stmt.condition.op),
+                self._expr(stmt.condition.value1),
+                self._expr(stmt.condition.value2),
+            )
 
             self.state.add_loop_exit_stmt(stmt.label, condition=cond)
 
@@ -139,7 +135,6 @@ class SootBlockProcessor:
     #
 
     def _handle_SootLocal(self, expr):
-
         local_name = expr.name
 
         # First try state.locals
@@ -158,17 +153,15 @@ class SootBlockProcessor:
         return expr
 
     def _handle_SootIntConstant(self, expr):
-
         return expr.value
 
     def _handle_SootInterfaceInvokeExpr(self, expr):
-
         full_method = expr.class_name + "." + expr.method_name
 
         mapping = {
-            'java.util.Set.iterator': VariableTypes.Iterator,
-            'java.util.Iterator.hasNext': VariableTypes.HasNext,
-            'java.util.Iterator.next': VariableTypes.Next,
+            "java.util.Set.iterator": VariableTypes.Iterator,
+            "java.util.Iterator.hasNext": VariableTypes.HasNext,
+            "java.util.Iterator.next": VariableTypes.Next,
         }
 
         base_var = self._expr(expr.base)
@@ -178,6 +171,12 @@ class SootBlockProcessor:
 
         # Try to annotate the variable when applicable
         try:
+            try:
+                from pysoot.sootir.soot_value import SootValue
+            except ImportError:
+                l.error("Please install PySoot before analyzing Java byte code.")
+                raise
+
             var_type = mapping[full_method]
 
             if isinstance(base_var, (SootValue, AnnotatedVariable)):
@@ -192,18 +191,10 @@ class SootBlockProcessor:
 
 class LoopAnalysisState:
     def __init__(self, block):
-
-        # Delayed import
-        try:
-            from pysoot.sootir.soot_value import SootValue, SootLocal
-        except ImportError:
-            l.error("Please install PySoot before analyzing Java byte code.")
-            raise
-
         self.block = block
 
-        self.induction_variables = { }
-        self.locals = { }
+        self.induction_variables = {}
+        self.locals = {}
 
         self.loop_exit_stmts = set()
 
@@ -211,7 +202,6 @@ class LoopAnalysisState:
         return "<LoopAnalysisState %s>" % self.block.addr
 
     def copy(self):
-
         state = LoopAnalysisState(block=self.block)
 
         state.induction_variables = self.induction_variables.copy()
@@ -221,7 +211,6 @@ class LoopAnalysisState:
         return state
 
     def merge(self, state):
-
         s = self.copy()
 
         # TODO: Induction variables
@@ -240,11 +229,10 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
     Analyze a loop and recover important information about the loop (e.g., invariants, induction variables) in a static
     manner.
     """
-    def __init__(self, loop, defuse):
 
+    def __init__(self, loop, defuse):
         visitor = LoopVisitor(loop)
-        ForwardAnalysis.__init__(self, order_jobs=True, allow_merging=True, allow_widening=False,
-                                 graph_visitor=visitor)
+        ForwardAnalysis.__init__(self, order_jobs=True, allow_merging=True, allow_widening=False, graph_visitor=visitor)
 
         self.loop = loop
         self.defuse = defuse
@@ -268,12 +256,10 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
         pass
 
     def _initial_abstract_state(self, node):
-
         state = LoopAnalysisState(node)
         return state
 
     def _merge_states(self, node, *states):
-
         merged = states[0]
         for other in states[1:]:
             if other is not None:
@@ -281,7 +267,6 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
         return merged
 
     def _run_on_node(self, node, state):
-
         if node in self._traversed:
             return False, state
 
@@ -300,7 +285,6 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
         pass
 
     def _post_analysis(self):
-
         self.loop_exit_stmts = self._last_state.loop_exit_stmts
         self.locals = self._last_state.locals
 
@@ -334,15 +318,17 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
         """
 
         # Condition 0
-        check_0 = lambda cond: (isinstance(cond, Condition) and
-                                cond.op == Condition.Equal and
-                                cond.val1 == 0 and
-                                isinstance(cond.val0, AnnotatedVariable) and
-                                cond.val0.type == VariableTypes.HasNext
-                                )
+        def check_0(cond):
+            return (
+                isinstance(cond, Condition)
+                and cond.op == Condition.Equal
+                and cond.val1 == 0
+                and isinstance(cond.val0, AnnotatedVariable)
+                and cond.val0.type == VariableTypes.HasNext
+            )
 
-        check_0_results = [ (check_0(stmt[0]), stmt[0]) for stmt in self.loop_exit_stmts ]
-        check_0_conds = [ cond for r, cond in check_0_results if r ]  # remove all False ones
+        check_0_results = [(check_0(stmt[0]), stmt[0]) for stmt in self.loop_exit_stmts]
+        check_0_conds = [cond for r, cond in check_0_results if r]  # remove all False ones
 
         if not check_0_conds:
             return None
@@ -350,15 +336,17 @@ class LoopAnalysis(ForwardAnalysis, Analysis):
         the_iterator = check_0_conds[0].val0.variable
 
         # Condition 1
-        check_1 = lambda local: (isinstance(local, AnnotatedVariable) and
-                                 local.type == VariableTypes.Next and
-                                 local.variable == the_iterator
-                                 )
+        def check_1(local):
+            return (
+                isinstance(local, AnnotatedVariable)
+                and local.type == VariableTypes.Next
+                and local.variable == the_iterator
+            )
 
-        if not any([ check_1(local) for local in self.locals.values() ]):
+        if not any([check_1(local) for local in self.locals.values()]):
             return None
 
         return True
 
 
-register_analysis(LoopAnalysis, 'LoopAnalysis')
+register_analysis(LoopAnalysis, "LoopAnalysis")
