@@ -1,12 +1,13 @@
 # pylint:disable=isinstance-second-argument-not-valid-type
 import weakref
-from typing import Set, Optional, Any, Tuple, Union, List
+from typing import Set, Optional, Any, Tuple, Union, List, DefaultDict
 from collections import defaultdict
 import logging
 
 import claripy
 import ailment
 import pyvex
+from archinfo.arch_arm import is_arm_arch
 
 from ... import sim_options
 from ...storage.memory_mixins import LabeledMemory
@@ -29,13 +30,32 @@ class PropagatorState:
     Describes the base state used in Propagator.
     """
 
-    __slots__ = ('arch', 'gpr_size', '_prop_count', '_only_consts', '_replacements', '_equivalence', 'project',
-                 '_store_tops', '_gp', '__weakref__', )
+    __slots__ = (
+        "arch",
+        "gpr_size",
+        "_prop_count",
+        "_only_consts",
+        "_replacements",
+        "_equivalence",
+        "project",
+        "_store_tops",
+        "_gp",
+        "__weakref__",
+    )
 
     _tops = {}
 
-    def __init__(self, arch, project=None, replacements=None, only_consts=False, prop_count=None, equivalence=None,
-                 store_tops=True, gp=None):
+    def __init__(
+        self,
+        arch,
+        project=None,
+        replacements=None,
+        only_consts=False,
+        prop_count=None,
+        equivalence=None,
+        store_tops=True,
+        gp=None,
+    ):
         self.arch = arch
         self.gpr_size = arch.bits // arch.byte_width  # size of the general-purpose registers
 
@@ -58,9 +78,24 @@ class PropagatorState:
         return weakref.proxy(self)
 
     @staticmethod
-    def _mo_cmp(mo_self: Union['SimMemoryObject','SimLabeledMemoryObject'],
-                mo_other: Union['SimMemoryObject','SimLabeledMemoryObject'],
-                addr: int, size: int):  # pylint:disable=unused-argument
+    def _is_const(v) -> bool:
+        if isinstance(v, (int, ailment.Expr.Const)):
+            return True
+        if isinstance(v, claripy.ast.BV) and v.op == "BVV":
+            return True
+        if isinstance(v, claripy.ast.FP) and v.op == "FPV":
+            return True
+        if isinstance(v, claripy.ast.Bool) and v.op == "BoolV":
+            return True
+        return False
+
+    @staticmethod
+    def _mo_cmp(
+        mo_self: Union["SimMemoryObject", "SimLabeledMemoryObject"],
+        mo_other: Union["SimMemoryObject", "SimLabeledMemoryObject"],
+        addr: int,
+        size: int,
+    ):  # pylint:disable=unused-argument
         # comparing bytes from two sets of memory objects
         # we don't need to resort to byte-level comparison. object-level is good enough.
 
@@ -74,7 +109,7 @@ class PropagatorState:
         return None
 
     @staticmethod
-    def top(bits: int) -> claripy.ast.Base:
+    def top(bits: int) -> claripy.ast.Bits:
         """
         Get a TOP value.
 
@@ -103,11 +138,10 @@ class PropagatorState:
                 return True
         return False
 
-    def copy(self) -> 'PropagatorState':
+    def copy(self) -> "PropagatorState":
         raise NotImplementedError()
 
     def merge(self, *others):
-
         state = self.copy()
         merge_occurred = False
 
@@ -151,7 +185,7 @@ class PropagatorState:
             return
 
         if self._only_consts:
-            if isinstance(new, int) or self.is_top(new):
+            if self._is_const(new) or self.is_top(new):
                 self._replacements[codeloc][old] = new
         else:
             self._replacements[codeloc][old] = new
@@ -162,28 +196,51 @@ class PropagatorState:
 
 # VEX state
 
+
 class PropagatorVEXState(PropagatorState):
     """
     Describes the state used in the VEX engine of Propagator.
     """
 
-    __slots__ = ('_registers', '_stack_variables', 'do_binops', )
+    __slots__ = (
+        "_registers",
+        "_stack_variables",
+        "do_binops",
+    )
 
-    def __init__(self, arch, project=None, registers=None, local_variables=None, replacements=None, only_consts=False,
-                 prop_count=None, do_binops=True, store_tops=True, gp=None):
-        super().__init__(arch, project=project, replacements=replacements, only_consts=only_consts,
-                         prop_count=prop_count, store_tops=store_tops, gp=gp)
+    def __init__(
+        self,
+        arch,
+        project=None,
+        registers=None,
+        local_variables=None,
+        replacements=None,
+        only_consts=False,
+        prop_count=None,
+        do_binops=True,
+        store_tops=True,
+        gp=None,
+    ):
+        super().__init__(
+            arch,
+            project=project,
+            replacements=replacements,
+            only_consts=only_consts,
+            prop_count=prop_count,
+            store_tops=store_tops,
+            gp=gp,
+        )
         self.do_binops = do_binops
-        self._registers = LabeledMemory(
-            memory_id='reg',
-            top_func=self.top,
-            page_kwargs={'mo_cmp': self._mo_cmp}) \
-            if registers is None else registers
-        self._stack_variables = LabeledMemory(
-            memory_id='mem',
-            top_func=self.top,
-            page_kwargs={'mo_cmp': self._mo_cmp}) \
-            if local_variables is None else local_variables
+        self._registers = (
+            LabeledMemory(memory_id="reg", top_func=self.top, page_kwargs={"mo_cmp": self._mo_cmp})
+            if registers is None
+            else registers
+        )
+        self._stack_variables = (
+            LabeledMemory(memory_id="mem", top_func=self.top, page_kwargs={"mo_cmp": self._mo_cmp})
+            if local_variables is None
+            else local_variables
+        )
 
         self._registers.set_state(self)
         self._stack_variables.set_state(self)
@@ -191,7 +248,7 @@ class PropagatorVEXState(PropagatorState):
     def __repr__(self):
         return "<PropagatorVEXState>"
 
-    def copy(self) -> 'PropagatorVEXState':
+    def copy(self) -> "PropagatorVEXState":
         cp = PropagatorVEXState(
             self.arch,
             project=self.project,
@@ -207,7 +264,7 @@ class PropagatorVEXState(PropagatorState):
 
         return cp
 
-    def merge(self, *others: 'PropagatorVEXState') -> Tuple['PropagatorVEXState',bool]:
+    def merge(self, *others: "PropagatorVEXState") -> Tuple["PropagatorVEXState", bool]:
         state = self.copy()
         merge_occurred = state._registers.merge([o._registers for o in others], None)
         merge_occurred |= state._stack_variables.merge([o._stack_variables for o in others], None)
@@ -228,7 +285,6 @@ class PropagatorVEXState(PropagatorState):
         self._registers.store(offset, value, size=size)
 
     def load_register(self, offset, size):
-
         # TODO: Fix me
         if size != self.gpr_size:
             return self.top(size * self.arch.byte_width)
@@ -247,7 +303,11 @@ class Equivalence:
     Describes an equivalence relationship between two atoms.
     """
 
-    __slots__ = ('codeloc', 'atom0', 'atom1',)
+    __slots__ = (
+        "codeloc",
+        "atom0",
+        "atom1",
+    )
 
     def __init__(self, codeloc, atom0, atom1):
         self.codeloc = codeloc
@@ -255,13 +315,15 @@ class Equivalence:
         self.atom1 = atom1
 
     def __repr__(self):
-        return "<Eq@%r: %r==%r>" % (self.codeloc, self.atom0, self.atom1)
+        return f"<Eq@{self.codeloc!r}: {self.atom0!r}=={self.atom1!r}>"
 
     def __eq__(self, other):
-        return type(other) is Equivalence \
-               and other.codeloc == self.codeloc \
-               and other.atom0 == self.atom0 \
-               and other.atom1 == self.atom1
+        return (
+            type(other) is Equivalence
+            and other.codeloc == self.codeloc
+            and other.atom0 == self.atom0
+            and other.atom1 == self.atom1
+        )
 
     def __hash__(self):
         return hash((Equivalence, self.codeloc, self.atom0, self.atom1))
@@ -272,36 +334,69 @@ class PropagatorAILState(PropagatorState):
     Describes the state used in the AIL engine of Propagator.
     """
 
-    __slots__ = ('_registers', '_stack_variables', '_tmps', 'temp_expressions', 'register_expressions',
-                 'last_stack_store', 'global_stores')
+    __slots__ = (
+        "_registers",
+        "_stack_variables",
+        "_tmps",
+        "temp_expressions",
+        "register_expressions",
+        "last_stack_store",
+        "global_stores",
+        "block_initial_reg_values",
+    )
 
-    def __init__(self, arch, project=None, replacements=None, only_consts=False, prop_count=None, equivalence=None,
-                 stack_variables=None, registers=None, gp=None):
-        super().__init__(arch, project=project, replacements=replacements, only_consts=only_consts,
-                         prop_count=prop_count, equivalence=equivalence, gp=gp)
+    def __init__(
+        self,
+        arch,
+        project=None,
+        replacements=None,
+        only_consts=False,
+        prop_count=None,
+        equivalence=None,
+        stack_variables=None,
+        registers=None,
+        gp=None,
+        block_initial_reg_values=None,
+    ):
+        super().__init__(
+            arch,
+            project=project,
+            replacements=replacements,
+            only_consts=only_consts,
+            prop_count=prop_count,
+            equivalence=equivalence,
+            gp=gp,
+        )
 
-        self._stack_variables = LabeledMemory(memory_id='mem',
-                                              top_func=self.top,
-                                              page_kwargs={'mo_cmp': self._mo_cmp}) \
-            if stack_variables is None else stack_variables
-        self._registers = LabeledMemory(memory_id='reg', top_func=self.top, page_kwargs={'mo_cmp': self._mo_cmp}) \
-            if registers is None else registers
+        self._stack_variables = (
+            LabeledMemory(memory_id="mem", top_func=self.top, page_kwargs={"mo_cmp": self._mo_cmp})
+            if stack_variables is None
+            else stack_variables
+        )
+        self._registers = (
+            LabeledMemory(memory_id="reg", top_func=self.top, page_kwargs={"mo_cmp": self._mo_cmp})
+            if registers is None
+            else registers
+        )
         self._tmps = {}
-        self.temp_expressions = { }
-        self.register_expressions = { }
+        self.temp_expressions = {}
+        self.register_expressions = {}
+        self.block_initial_reg_values: DefaultDict[
+            Tuple[int, int], List[Tuple[ailment.Expr.Register, ailment.Expr.Const]]
+        ] = (defaultdict(list) if block_initial_reg_values is None else block_initial_reg_values)
 
         self._registers.set_state(self)
         self._stack_variables.set_state(self)
         # last_stack_store stores the most recent stack store statement with a non-concrete or unresolvable address. we
         # use this information to determine if stack reads after this store can be safely resolved to definitions prior
         # to the stack read.
-        self.last_stack_store: Optional[Tuple[int,int,ailment.Stmt.Store]] = None
-        self.global_stores: List[Tuple[int,int,Any,ailment.Stmt.Store]] = [ ]
+        self.last_stack_store: Optional[Tuple[int, int, ailment.Stmt.Store]] = None
+        self.global_stores: List[Tuple[int, int, Any, ailment.Stmt.Store]] = []
 
     def __repr__(self):
         return "<PropagatorAILState>"
 
-    def copy(self):
+    def copy(self) -> "PropagatorAILState":
         rd = PropagatorAILState(
             self.arch,
             project=self.project,
@@ -311,15 +406,35 @@ class PropagatorAILState(PropagatorState):
             equivalence=self._equivalence.copy(),
             stack_variables=self._stack_variables.copy(),
             registers=self._registers.copy(),
+            block_initial_reg_values=self.block_initial_reg_values.copy(),
             # drop tmps
             gp=self._gp,
         )
 
         return rd
 
-    def merge(self, *others) -> Tuple['PropagatorAILState',bool]:
+    @staticmethod
+    def is_const_or_register(value: Optional[Union[ailment.Expr.Expression, claripy.ast.Bits]]) -> bool:
+        if value is None:
+            return False
+        if isinstance(value, claripy.ast.BV):
+            return not value.symbolic
+        if isinstance(value, ailment.Expr.Register):
+            return True
+        if isinstance(value, ailment.Expr.Const) or (isinstance(value, int) and value == 0):
+            return True
+        # more hacks: also store the eq comparisons
+        if isinstance(value, ailment.Expr.BinaryOp) and value.op == "CmpEQ":
+            if all(isinstance(arg, (ailment.Expr.Const, ailment.Expr.Tmp)) for arg in value.operands):
+                return True
+        # more hacks: also store the conversions
+        if isinstance(value, ailment.Expr.Convert) and PropagatorAILState.is_const_or_register(value.operand):
+            return True
+        return False
+
+    def merge(self, *others) -> Tuple["PropagatorAILState", bool]:
         state, merge_occurred = super().merge(*others)
-        state: 'PropagatorAILState'
+        state: "PropagatorAILState"
 
         merge_occurred |= state._registers.merge([o._registers for o in others], None)
         merge_occurred |= state._stack_variables.merge([o._stack_variables for o in others], None)
@@ -337,10 +452,17 @@ class PropagatorAILState(PropagatorState):
             return
 
         for offset, chopped_value, size, label in value.value_and_labels():
-            self._registers.store(reg.reg_offset + offset, chopped_value, size=size, label=label,
-                                  endness=self.project.arch.register_endness)
+            self._registers.store(
+                reg.reg_offset + offset,
+                chopped_value,
+                size=size,
+                label=label,
+                endness=self.project.arch.register_endness,
+            )
 
-    def store_stack_variable(self, sp_offset: int, new: PropValue, endness=None) -> None:  # pylint:disable=unused-argument
+    def store_stack_variable(
+        self, sp_offset: int, new: PropValue, endness=None
+    ) -> None:  # pylint:disable=unused-argument
         # normalize sp_offset to handle negative offsets
         sp_offset += 0x65536
         sp_offset &= (1 << self.arch.bits) - 1
@@ -350,8 +472,9 @@ class PropagatorAILState(PropagatorState):
 
     def load_register(self, reg: ailment.Expr.Register) -> Optional[PropValue]:
         try:
-            value, labels = self._registers.load_with_labels(reg.reg_offset, size=reg.size,
-                                                             endness=self.project.arch.register_endness)
+            value, labels = self._registers.load_with_labels(
+                reg.reg_offset, size=reg.size, endness=self.project.arch.register_endness
+            )
         except SimMemoryMissingError:
             # value does not exist
             return None
@@ -370,9 +493,9 @@ class PropagatorAILState(PropagatorState):
             if ex.missing_addr > sp_offset:
                 # some data exist. load again
                 try:
-                    value, labels = self._stack_variables.load_with_labels(sp_offset,
-                                                                           size=ex.missing_addr - sp_offset,
-                                                                           endness=endness)
+                    value, labels = self._stack_variables.load_with_labels(
+                        sp_offset, size=ex.missing_addr - sp_offset, endness=endness
+                    )
                     # then we zero-extend both the value and labels
                     if value is not None and len(labels) == 1 and labels[0][0] == 0:
                         value = claripy.ZeroExt(ex.missing_size * self.arch.byte_width, value)
@@ -388,6 +511,11 @@ class PropagatorAILState(PropagatorState):
         return prop_value
 
     def add_replacement(self, codeloc, old, new):
+        if self._only_consts:
+            if self.is_const_or_register(new) or self.is_top(new):
+                pass
+            else:
+                new = self.top(1)
 
         # do not replace anything with a call expression
         if isinstance(new, ailment.statement.Call):
@@ -407,13 +535,19 @@ class PropagatorAILState(PropagatorState):
             return
 
         prop_count = 0
-        if not isinstance(old, ailment.Expr.Tmp) and isinstance(new, ailment.Expr.Expression) \
-                and not isinstance(new, ailment.Expr.Const):
+        if (
+            not isinstance(old, ailment.Expr.Tmp)
+            and isinstance(new, ailment.Expr.Expression)
+            and not isinstance(new, ailment.Expr.Const)
+        ):
             self._prop_count[new] += 1
             prop_count = self._prop_count[new]
 
-        if prop_count <= 1 or isinstance(new, ailment.Expr.StackBaseOffset) or (
-                isinstance(old, ailment.Expr.Register) and self.arch.is_artificial_register(old.reg_offset, old.size)):
+        if (
+            prop_count <= 1
+            or isinstance(new, ailment.Expr.StackBaseOffset)
+            or (isinstance(old, ailment.Expr.Register) and self.arch.is_artificial_register(old.reg_offset, old.size))
+        ):
             # we can propagate this expression
             super().add_replacement(codeloc, old, new)
         else:
@@ -423,7 +557,6 @@ class PropagatorAILState(PropagatorState):
                     self._replacements[codeloc_][old] = self.top(1)
 
     def filter_replacements(self):
-
         to_remove = set()
 
         for old, new in self._replacements.items():
@@ -462,10 +595,23 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
     - Writing values to a stack variable
     """
 
-    def __init__(self, func=None, block=None, func_graph=None, base_state=None, max_iterations=3,
-                 load_callback=None, stack_pointer_tracker=None, only_consts=False, completed_funcs=None,
-                 do_binops=True, store_tops=True, vex_cross_insn_opt=False, func_addr: Optional[int]=None,
-                 gp: Optional[int]=None):
+    def __init__(
+        self,
+        func=None,
+        block=None,
+        func_graph=None,
+        base_state=None,
+        max_iterations=3,
+        load_callback=None,
+        stack_pointer_tracker=None,
+        only_consts=False,
+        completed_funcs=None,
+        do_binops=True,
+        store_tops=True,
+        vex_cross_insn_opt=False,
+        func_addr: Optional[int] = None,
+        gp: Optional[int] = None,
+    ):
         if block is None and func is not None:
             # only func is specified. traversing a function
             graph_visitor = FunctionGraphVisitor(func, func_graph)
@@ -474,10 +620,11 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
             # value for register t9 for MIPS32/64 binaries)
             graph_visitor = SingleNodeGraphVisitor(block)
         else:
-            raise ValueError('Unsupported analysis target.')
+            raise ValueError("Unsupported analysis target.")
 
-        ForwardAnalysis.__init__(self, order_jobs=True, allow_merging=True, allow_widening=False,
-                                 graph_visitor=graph_visitor)
+        ForwardAnalysis.__init__(
+            self, order_jobs=True, allow_merging=True, allow_widening=False, graph_visitor=graph_visitor
+        )
 
         self._base_state = base_state
         self._function = func
@@ -494,6 +641,7 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
 
         self._node_iterations = defaultdict(int)
         self._states = {}
+        self._block_initial_reg_values = {}
         self.replacements: Optional[defaultdict] = None
         self.equivalence: Set[Equivalence] = set()
 
@@ -514,7 +662,7 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
     # Main analysis routines
     #
 
-    def _node_key(self, node: Union[ailment.Block,pyvex.IRSB]) -> Any:
+    def _node_key(self, node: Union[ailment.Block, pyvex.IRSB]) -> Any:
         if type(node) is ailment.Block:
             return node.addr, node.idx
         elif type(node) is pyvex.IRSB:
@@ -531,51 +679,92 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
     def _initial_abstract_state(self, node):
         if isinstance(node, ailment.Block):
             # AIL
-            state = PropagatorAILState(self.project.arch, project=self.project, only_consts=self._only_consts,
-                                       gp=self._gp)
+            state = PropagatorAILState(
+                self.project.arch, project=self.project, only_consts=self._only_consts, gp=self._gp
+            )
             ail = True
+            spoffset_var = ailment.Expr.StackBaseOffset(None, self.project.arch.bits, 0)
+            sp_value = PropValue(
+                claripy.BVV(0x7FFF_FF00, self.project.arch.bits),
+                offset_and_details={0: Detail(self.project.arch.bytes, spoffset_var, None)},
+            )
+            state.store_register(
+                ailment.Expr.Register(None, None, self.project.arch.sp_offset, self.project.arch.bits),
+                sp_value,
+            )
         else:
             # VEX
-            state = PropagatorVEXState(self.project.arch, project=self.project, only_consts=self._only_consts,
-                                       do_binops=self._do_binops, store_tops=self._store_tops,
-                                       gp=self._gp)
+            state = PropagatorVEXState(
+                self.project.arch,
+                project=self.project,
+                only_consts=self._only_consts,
+                do_binops=self._do_binops,
+                store_tops=self._store_tops,
+                gp=self._gp,
+            )
             spoffset_var = self._engine_vex.sp_offset(0)
             ail = False
-            state.store_register(self.project.arch.sp_offset,
-                                 self.project.arch.bytes,
-                                 spoffset_var,
-                                 )
+            state.store_register(
+                self.project.arch.sp_offset,
+                self.project.arch.bytes,
+                spoffset_var,
+            )
 
         if self.project.arch.name == "MIPS64":
             if self._func_addr is not None:
                 if ail:
-                    reg_expr = ailment.Expr.Register(None, None, self.project.arch.registers['t9'][0],
-                                                     self.project.arch.registers['t9'][1])
+                    reg_expr = ailment.Expr.Register(
+                        None, None, self.project.arch.registers["t9"][0], self.project.arch.registers["t9"][1]
+                    )
                     reg_value = ailment.Expr.Const(None, None, self._func_addr, 64)
-                    state.store_register(reg_expr,
-                                         PropValue(claripy.BVV(self._func_addr, 64),
-                                                   offset_and_details={0: Detail(8, reg_value, CodeLocation(0, 0))}),
-                                         )
+                    state.store_register(
+                        reg_expr,
+                        PropValue(
+                            claripy.BVV(self._func_addr, 64),
+                            offset_and_details={0: Detail(8, reg_value, CodeLocation(0, 0))},
+                        ),
+                    )
                 else:
-                    state.store_register(self.project.arch.registers['t9'][0],  # pylint:disable=too-many-function-args
-                                         self.project.arch.registers['t9'][1],
-                                         claripy.BVV(self._func_addr, 64),
-                                         )
+                    state.store_register(  # pylint:disable=too-many-function-args
+                        self.project.arch.registers["t9"][0],
+                        self.project.arch.registers["t9"][1],
+                        claripy.BVV(self._func_addr, 64),
+                    )
         elif self.project.arch.name == "MIPS32":
             if self._func_addr is not None:
                 if ail:
-                    reg_expr = ailment.Expr.Register(None, None, self.project.arch.registers['t9'][0],
-                                                                 self.project.arch.registers['t9'][1])
+                    reg_expr = ailment.Expr.Register(
+                        None, None, self.project.arch.registers["t9"][0], self.project.arch.registers["t9"][1]
+                    )
                     reg_value = ailment.Expr.Const(None, None, self._func_addr, 32)
-                    state.store_register(reg_expr,
-                                         PropValue(claripy.BVV(self._func_addr, 32),
-                                                   offset_and_details={0: Detail(4, reg_value, CodeLocation(0, 0))}),
-                                         )
+                    state.store_register(
+                        reg_expr,
+                        PropValue(
+                            claripy.BVV(self._func_addr, 32),
+                            offset_and_details={0: Detail(4, reg_value, CodeLocation(0, 0))},
+                        ),
+                    )
                 else:
-                    state.store_register(self.project.arch.registers['t9'][0],  # pylint:disable=too-many-function-args
-                                         self.project.arch.registers['t9'][1],
-                                         claripy.BVV(self._func_addr, 32),
-                                         )
+                    state.store_register(  # pylint:disable=too-many-function-args
+                        self.project.arch.registers["t9"][0],
+                        self.project.arch.registers["t9"][1],
+                        claripy.BVV(self._func_addr, 32),
+                    )
+        elif is_arm_arch(self.project.arch):
+            if ail:
+                # clear fpscr
+                reg_expr = ailment.Expr.Register(None, None, *self.project.arch.registers["fpscr"])
+                reg_value = ailment.Expr.Const(None, None, 0, 32)
+                state.store_register(
+                    reg_expr,
+                    PropValue(claripy.BVV(0, 32), offset_and_details={0: Detail(4, reg_value, CodeLocation(0, 0))}),
+                )
+            else:
+                state.store_register(  # pylint:disable=too-many-function-args
+                    self.project.arch.registers["fpscr"][0],
+                    self.project.arch.registers["fpscr"][1],
+                    claripy.BVV(0, 32),
+                )
 
         self._initial_state = state
         return state
@@ -585,14 +774,14 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         return merged_state, not merge_occurred
 
     def _run_on_node(self, node, state):
-
         if isinstance(node, ailment.Block):
             block = node
             block_key = (node.addr, node.idx)
             engine = self._engine_ail
         else:
-            block = self.project.factory.block(node.addr, node.size, opt_level=1,
-                                               cross_insn_opt=self._vex_cross_insn_opt)
+            block = self.project.factory.block(
+                node.addr, node.size, opt_level=1, cross_insn_opt=self._vex_cross_insn_opt
+            )
             block_key = node.addr
             engine = self._engine_vex
             if block.size == 0:
@@ -610,17 +799,25 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         if self._base_state is not None:
             self._base_state.options.add(sim_options.SYMBOL_FILL_UNCONSTRAINED_REGISTERS)
             self._base_state.options.add(sim_options.SYMBOL_FILL_UNCONSTRAINED_MEMORY)
-        state = engine.process(state, block=block, project=self.project, base_state=self._base_state,
-                               load_callback=self._load_callback, fail_fast=self._fail_fast)
+        state = engine.process(
+            state,
+            block=block,
+            project=self.project,
+            base_state=self._base_state,
+            load_callback=self._load_callback,
+            fail_fast=self._fail_fast,
+        )
         state.filter_replacements()
 
         self._node_iterations[block_key] += 1
         self._states[block_key] = state
+        if isinstance(state, PropagatorAILState):
+            self._block_initial_reg_values.update(state.block_initial_reg_values)
 
         if self.replacements is None:
             self.replacements = state._replacements
         else:
-            self.replacements.update(state._replacements)
+            self._merge_replacements(self.replacements, state._replacements)
 
         self.equivalence |= state._equivalence
 
@@ -631,6 +828,24 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         else:
             return False, state
 
+    def _process_input_state_for_successor(
+        self, node, successor, input_state: Union[PropagatorAILState, PropagatorVEXState]
+    ):
+        if self._only_consts and isinstance(input_state, PropagatorAILState):
+            key = node.addr, successor.addr
+            if key in self._block_initial_reg_values:
+                input_state: PropagatorAILState = input_state.copy()
+                for reg_atom, reg_value in self._block_initial_reg_values[key]:
+                    input_state.store_register(
+                        reg_atom,
+                        PropValue(
+                            claripy.BVV(reg_value.value, reg_value.bits),
+                            offset_and_details={0: Detail(reg_atom.size, reg_value, None)},
+                        ),
+                    )
+                return input_state
+        return input_state
+
     def _intra_analysis(self):
         pass
 
@@ -662,7 +877,7 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
         # Filter replacements and remove all TOP values
         if self.replacements is not None:
             for codeloc in list(self.replacements.keys()):
-                rep = dict((k, v) for k, v in self.replacements[codeloc].items() if not PropagatorState.is_top(v))
+                rep = {k: v for k, v in self.replacements[codeloc].items() if not PropagatorState.is_top(v)}
                 self.replacements[codeloc] = rep
 
         if self._function is not None:
@@ -714,5 +929,38 @@ class PropagatorAnalysis(ForwardAnalysis, Analysis):  # pylint:disable=abstract-
 
         self._post_analysis()
 
+    @staticmethod
+    def _merge_replacements(replacements_0, replacements_1) -> bool:
+        merge_occurred = False
+        for loc, vars_ in replacements_1.items():
+            if loc not in replacements_0:
+                replacements_0[loc] = vars_.copy()
+                merge_occurred = True
+            else:
+                for var, repl in vars_.items():
+                    if var not in replacements_0[loc]:
+                        replacements_0[loc][var] = repl
+                        merge_occurred = True
+                    else:
+                        if PropagatorState.is_top(repl) or PropagatorState.is_top(replacements_0[loc][var]):
+                            t = PropagatorState.top(repl.bits if isinstance(repl, ailment.Expression) else repl.size())
+                            replacements_0[loc][var] = t
+                            merge_occurred = True
+                        elif (
+                            isinstance(replacements_0[loc][var], claripy.ast.Base) or isinstance(repl, claripy.ast.Base)
+                        ) and replacements_0[loc][var] is not repl:
+                            t = PropagatorState.top(repl.bits if isinstance(repl, ailment.Expression) else repl.size())
+                            replacements_0[loc][var] = t
+                            merge_occurred = True
+                        elif (
+                            not isinstance(replacements_0[loc][var], claripy.ast.Base)
+                            and not isinstance(repl, claripy.ast.Base)
+                            and replacements_0[loc][var] != repl
+                        ):
+                            t = PropagatorState.top(repl.bits if isinstance(repl, ailment.Expression) else repl.size())
+                            replacements_0[loc][var] = t
+                            merge_occurred = True
+        return merge_occurred
+
 
 register_analysis(PropagatorAnalysis, "Propagator")
