@@ -7,10 +7,27 @@ class CodeLocation:
     name (for SimProcedures).
     """
 
-    __slots__ = ('block_addr', 'stmt_idx', 'sim_procedure', 'ins_addr', 'context', 'info', 'block_idx', '_hash', )
+    __slots__ = (
+        "block_addr",
+        "stmt_idx",
+        "sim_procedure",
+        "ins_addr",
+        "context",
+        "info",
+        "block_idx",
+        "_hash",
+    )
 
-    def __init__(self, block_addr: int, stmt_idx: Optional[int], sim_procedure=None, ins_addr: Optional[int]=None,
-                 context: Optional[Tuple[int]]=None, block_idx: int=None, **kwargs):
+    def __init__(
+        self,
+        block_addr: int,
+        stmt_idx: Optional[int],
+        sim_procedure=None,
+        ins_addr: Optional[int] = None,
+        context: Optional[Tuple[int]] = None,
+        block_idx: int = None,
+        **kwargs,
+    ):
         """
         Constructor.
 
@@ -38,10 +55,10 @@ class CodeLocation:
 
     def __repr__(self):
         if self.block_addr is None:
-            return '<%s>' % self.sim_procedure
+            return "<%s>" % self.sim_procedure
 
         if self.stmt_idx is None:
-            s = "<%s%#x(-)" % (
+            s = "<{}{:#x}(-)".format(
                 ("%#x " % self.ins_addr) if self.ins_addr else "",
                 self.block_addr,
             )
@@ -55,13 +72,13 @@ class CodeLocation:
         if self.context is None:
             s += " contextless"
         else:
-            s += " context: %r" % (self.context,)
+            s += f" context: {self.context!r}"
 
-        ss = [ ]
+        ss = []
         if self.info:
             for k, v in self.info.items():
-                if v != tuple() and v is not None:
-                    ss.append("%s=%s" % (k, v))
+                if v != () and v is not None:
+                    ss.append(f"{k}={v}")
             if ss:
                 s += " with %s" % ", ".join(ss)
         s += ">"
@@ -79,11 +96,15 @@ class CodeLocation:
         """
         Check if self is the same as other.
         """
-        return type(self) is type(other) and self.block_addr == other.block_addr and \
-                self.stmt_idx == other.stmt_idx and self.sim_procedure is other.sim_procedure and \
-                self.context == other.context and \
-                self.block_idx == other.block_idx and \
-                self.ins_addr == other.ins_addr
+        return (
+            type(self) is type(other)
+            and self.block_addr == other.block_addr
+            and self.stmt_idx == other.stmt_idx
+            and self.sim_procedure is other.sim_procedure
+            and self.context == other.context
+            and self.block_idx == other.block_idx
+            and self.ins_addr == other.ins_addr
+        )
 
     def __lt__(self, other):
         if self.block_addr != other.block_addr:
@@ -100,12 +121,13 @@ class CodeLocation:
         returns the hash value of self.
         """
         if self._hash is None:
-            self._hash = hash((self.block_addr, self.stmt_idx, self.sim_procedure, self.ins_addr, self.context,
-                               self.block_idx))
+            self._hash = hash(
+                (self.block_addr, self.stmt_idx, self.sim_procedure, self.ins_addr, self.context, self.block_idx)
+            )
         return self._hash
 
     def _store_kwargs(self, **kwargs):
         if self.info is None:
-            self.info = { }
+            self.info = {}
         for k, v in kwargs.items():
             self.info[k] = v
