@@ -1,6 +1,6 @@
-
 from .functions import FunctionManager, Function
 from .variables import VariableManager
+from .debug_variables import DebugVariableManager
 from .comments import Comments
 from .data import Data
 from .indirect_jumps import IndirectJumps
@@ -14,3 +14,5 @@ from .key_definitions import KeyDefinitionManager
 from .propagations import Propagations
 from .structured_code import StructuredCodeManager
 from .types import TypesStore
+from .gotos import Gotos
+from .callsite_prototypes import CallsitePrototypes
