@@ -1,7 +1,6 @@
 from typing import Dict, Tuple, Union, Optional
 
 import claripy
-import ailment
 
 from ...calling_conventions import SimFunctionArgument, SimRegArg, SimStackArg
 from ...engines.light import SpOffset
@@ -16,7 +15,7 @@ class Atom:
     It could either be a Tmp (temporary variable), a Register, a MemoryLocation.
     """
 
-    __slots__ = ('_hash', )
+    __slots__ = ("_hash",)
 
     def __init__(self):
         self._hash = None
@@ -29,7 +28,7 @@ class Atom:
         raise NotImplementedError()
 
     @staticmethod
-    def from_argument(argument: SimFunctionArgument, registers: Dict[str,Tuple[int,int]]):
+    def from_argument(argument: SimFunctionArgument, registers: Dict[str, Tuple[int, int]]):
         """
         Instanciate an `Atom` from a given argument.
 
@@ -56,6 +55,7 @@ class GuardUse(Atom):
     """
     Implements a guard use.
     """
+
     __slots__ = ("target",)
 
     def __init__(self, target):
@@ -63,7 +63,7 @@ class GuardUse(Atom):
         self.target = target
 
     def __repr__(self):
-        return '<Guard %#x>' % self.target
+        return "<Guard %#x>" % self.target
 
     @property
     def size(self) -> int:
@@ -76,7 +76,11 @@ class GuardUse(Atom):
 
 
 class FunctionCall(Atom):
-    __slots__ = ('target', 'callsite')
+    """
+    Represents a function call.
+    """
+
+    __slots__ = ("target", "callsite")
 
     def __init__(self, target, callsite):
         super().__init__()
@@ -85,15 +89,20 @@ class FunctionCall(Atom):
 
     @property
     def single_target(self):
-        if type(self.target) is MultiValues and len(self.target.values) == 1 and 0 in self.target.values and \
-                len(self.target.values[0]) == 1 and next(iter(self.target.values[0])).op == 'BVV':
+        if (
+            type(self.target) is MultiValues
+            and len(self.target.values) == 1
+            and 0 in self.target.values
+            and len(self.target.values[0]) == 1
+            and next(iter(self.target.values[0])).op == "BVV"
+        ):
             return next(iter(self.target.values[0])).args[0]
         return None
 
     def __repr__(self):
         target = self.single_target
-        target_txt = hex(target) if target is not None else '(indirect)'
-        return '<Call %s>' % target_txt
+        target_txt = hex(target) if target is not None else "(indirect)"
+        return "<Call %s>" % target_txt
 
     def __eq__(self, other):
         return type(other) is FunctionCall and self.callsite == other.callsite
@@ -109,7 +118,11 @@ class FunctionCall(Atom):
 
 
 class ConstantSrc(Atom):
-    __slots__ = ('const',)
+    """
+    Represents a constant.
+    """
+
+    __slots__ = ("const",)
 
     def __init__(self, const):
         super().__init__()
@@ -135,7 +148,11 @@ class Tmp(Atom):
     """
     Represents a variable used by the IR to store intermediate values.
     """
-    __slots__ = ('tmp_idx', '_size', )
+
+    __slots__ = (
+        "tmp_idx",
+        "_size",
+    )
 
     def __init__(self, tmp_idx: int, size: int):
         super().__init__()
@@ -151,7 +168,7 @@ class Tmp(Atom):
     __hash__ = Atom.__hash__
 
     def _core_hash(self):
-        return hash(('tmp', self.tmp_idx))
+        return hash(("tmp", self.tmp_idx))
 
     @property
     def size(self) -> int:
@@ -169,7 +186,11 @@ class Register(Atom):
     :ivar int reg_offset:    The offset from the base to define its place in the memory bloc.
     :ivar int size:          The size, in number of bytes.
     """
-    __slots__ = ('reg_offset', '_size', )
+
+    __slots__ = (
+        "reg_offset",
+        "_size",
+    )
 
     def __init__(self, reg_offset: int, size: int):
         super().__init__()
@@ -181,14 +202,12 @@ class Register(Atom):
         return "<Reg %d<%d>>" % (self.reg_offset, self.size)
 
     def __eq__(self, other):
-        return type(other) is Register and \
-               self.reg_offset == other.reg_offset and \
-               self.size == other.size
+        return type(other) is Register and self.reg_offset == other.reg_offset and self.size == other.size
 
     __hash__ = Atom.__hash__
 
     def _core_hash(self):
-        return hash(('reg', self.reg_offset, self.size))
+        return hash(("reg", self.reg_offset, self.size))
 
     @property
     def bits(self) -> int:
@@ -206,25 +225,29 @@ class MemoryLocation(Atom):
     It is characterized by its address and its size.
     """
 
-    __slots__ = ('addr', '_size', 'endness', )
+    __slots__ = (
+        "addr",
+        "_size",
+        "endness",
+    )
 
-    def __init__(self, addr: Union[SpOffset,HeapAddress,int], size: int, endness: Optional[str]=None):
+    def __init__(self, addr: Union[SpOffset, HeapAddress, int], size: int, endness: Optional[str] = None):
         """
         :param int addr: The address of the beginning memory location slice.
         :param int size: The size of the represented memory location, in bytes.
         """
         super().__init__()
 
-        self.addr: Union[SpOffset,int,claripy.ast.BV] = addr
+        self.addr: Union[SpOffset, int, claripy.ast.BV] = addr
         self._size: int = size
         self.endness = endness
 
     def __repr__(self):
         address_format = hex(self.addr) if type(self.addr) is int else self.addr
-        stack_format = ' (stack)' if self.is_on_stack else ''
+        stack_format = " (stack)" if self.is_on_stack else ""
         size = "%d" % self.size if isinstance(self.size, int) else self.size
 
-        return "<Mem %s<%s>%s>" % (address_format, size, stack_format)
+        return f"<Mem {address_format}<{size}>{stack_format}>"
 
     @property
     def is_on_stack(self) -> bool:
@@ -246,19 +269,23 @@ class MemoryLocation(Atom):
         if isinstance(self.addr, int):
             return False
         elif isinstance(self.addr, SpOffset):
-            return not type(self.addr.offset) is int
+            return type(self.addr.offset) is not int
         return True
 
     def __eq__(self, other):
         # pylint:disable=isinstance-second-argument-not-valid-type
-        return type(other) is MemoryLocation and \
-               (
-                    self.addr is other.addr if isinstance(self.addr, (claripy.ast.BV)) else self.addr == other.addr
-               ) and \
-               self.size == other.size and \
-               self.endness == other.endness
+        return (
+            type(other) is MemoryLocation
+            and (
+                self.addr is other.addr
+                if (isinstance(self.addr, claripy.ast.BV) or isinstance(other.addr, claripy.ast.BV))
+                else self.addr == other.addr
+            )
+            and self.size == other.size
+            and self.endness == other.endness
+        )
 
     __hash__ = Atom.__hash__
 
     def _core_hash(self):
-        return hash(('mem', self.addr, self.size, self.endness))
+        return hash(("mem", self.addr, self.size, self.endness))
