@@ -8,7 +8,7 @@ from typing import Optional, Dict, Type, TYPE_CHECKING
 
 import itanium_demangler
 
-from ...sim_type import parse_cpp_file, SimTypeFunction, SimTypeFloat, SimTypeDouble
+from ...sim_type import parse_cpp_file, SimTypeFunction
 from ...calling_conventions import DEFAULT_CC
 from ...misc import autoimport
 from ...misc.ux import once
@@ -646,6 +646,7 @@ COMMON_LIBRARIES = {
     'cgc',
     # (mostly) Linux
     'glibc',
+    'gnulib',  # really just for .o files in coreutils
     'libstdcpp',
     'linux_kernel',
     'linux_loader',
