@@ -7,8 +7,9 @@ from ...errors import SimSolverError
 # fseek
 ######################################
 
+
 class fseek(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
 
     def run(self, file_ptr, offset, whence):
         # TODO: Support symbolic file_ptr, offset, and whence
@@ -20,15 +21,16 @@ class fseek(angr.SimProcedure):
             raise angr.SimProcedureError('multi-valued "whence" is not supported in fseek.')
 
         try:
-            whence = {0: 'start', 1: 'current', 2: 'end'}[whence]
+            whence = {0: "start", 1: "current", 2: "end"}[whence]
         except KeyError:
-            return -1 # EINVAL
+            return -1  # EINVAL
 
-        fd_offset = io_file_data_for_arch(self.state.arch)['fd']
+        fd_offset = io_file_data_for_arch(self.state.arch)["fd"]
         fd = self.state.mem[file_ptr + fd_offset].int.resolved
         simfd = self.state.posix.get_fd(fd)
         if simfd is None:
             return -1
-        return self.state.solver.If(simfd.seek(offset, whence), self.state.solver.BVV(0, self.arch.sizeof['int']), -1)
+        return self.state.solver.If(simfd.seek(offset, whence), self.state.solver.BVV(0, self.arch.sizeof["int"]), -1)
+
 
 fseeko = fseek
