@@ -6,11 +6,12 @@ from cle.backends.externs.simdata.io_file import io_file_data_for_arch
 # ftell
 ######################################
 
+
 class ftell(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
 
     def run(self, file_ptr):
-        fd_offset = io_file_data_for_arch(self.state.arch)['fd']
+        fd_offset = io_file_data_for_arch(self.state.arch)["fd"]
         fd = self.state.mem[file_ptr + fd_offset].int.resolved
         simfd = self.state.posix.get_fd(fd)
         if simfd is None:
@@ -20,4 +21,5 @@ class ftell(angr.SimProcedure):
             return -1
         return pos
 
+
 ftello = ftell
