@@ -8,9 +8,9 @@ l = logging.getLogger(name=__name__)
 # sprintf
 ######################################
 
-class sprintf(FormatParser):
 
-    #pylint:disable=arguments-differ
+class sprintf(FormatParser):
+    # pylint:disable=arguments-differ
 
     def run(self, dst_ptr, fmt):  # pylint:disable=unused-argument
         # The format str is at index 1
@@ -19,6 +19,8 @@ class sprintf(FormatParser):
         self.state.memory.store(dst_ptr, out_str)
 
         # place the terminating null byte
-        self.state.memory.store(dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, self.arch.byte_width))
+        self.state.memory.store(
+            dst_ptr + (out_str.size() // self.arch.byte_width), self.state.solver.BVV(0, self.arch.byte_width)
+        )
 
-        return out_str.size()//self.arch.byte_width
+        return out_str.size() // self.arch.byte_width
