@@ -4,8 +4,9 @@ import angr
 # Redirect the control flow to some other places
 ######################################
 
+
 class Redirect(angr.SimProcedure):
-    #pylint:disable=arguments-differ
+    # pylint:disable=arguments-differ
 
     NO_RET = True
 
@@ -15,4 +16,4 @@ class Redirect(angr.SimProcedure):
 
         self._custom_name = "Redirect to 0x%08x" % redirect_to
         # There is definitely no refs
-        self.add_successor(self.state, redirect_to, self.state.solver.true, 'Ijk_Boring')
+        self.add_successor(self.state, redirect_to, self.state.solver.true, "Ijk_Boring")
