@@ -4,14 +4,14 @@ import angr
 # Returns an unconstrained byte
 ######################################
 
-class ReturnUnconstrained(angr.SimProcedure):
 
+class ReturnUnconstrained(angr.SimProcedure):
     ARGS_MISMATCH = True
 
-    def run(self, *args, **kwargs): #pylint:disable=arguments-differ
-        #pylint:disable=attribute-defined-outside-init
+    def run(self, *args, **kwargs):  # pylint:disable=arguments-differ
+        # pylint:disable=attribute-defined-outside-init
 
-        return_val = kwargs.pop('return_val', None)
+        return_val = kwargs.pop("return_val", None)
         if return_val is None:
             # code duplicated to syscall_stub
             size = self.prototype.returnty.size
@@ -19,7 +19,9 @@ class ReturnUnconstrained(angr.SimProcedure):
             if size is NotImplemented:
                 o = None
             else:
-                o = self.state.solver.Unconstrained("unconstrained_ret_%s" % self.display_name, size, key=('api', '?', self.display_name))
+                o = self.state.solver.Unconstrained(
+                    "unconstrained_ret_%s" % self.display_name, size, key=("api", "?", self.display_name)
+                )
         else:
             o = return_val
 
