@@ -18,6 +18,7 @@ from .errors import AngrNoPluginError
 
 l = logging.getLogger(name=__name__)
 
+
 def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False, **kwargs):
     """
     Load a new project based on a snippet of assembly or bytecode.
@@ -36,15 +37,15 @@ def load_shellcode(shellcode, arch, start_offset=0, load_address=0, thumb=False,
         start_offset |= 1
 
     return Project(
-            BytesIO(shellcode),
-            main_opts={
-                'backend': 'blob',
-                'arch': arch,
-                'entry_point': start_offset,
-                'base_addr': load_address,
-            },
-        **kwargs
-        )
+        BytesIO(shellcode),
+        main_opts={
+            "backend": "blob",
+            "arch": arch,
+            "entry_point": start_offset,
+            "base_addr": load_address,
+        },
+        **kwargs,
+    )
 
 
 class Project:
@@ -68,10 +69,9 @@ class Project:
     :param simos:                       a SimOS class to use for this project.
     :param engine:                      The SimEngine class to use for this project.
     :param bool translation_cache:      If True, cache translated basic blocks rather than re-translating them.
-    :param support_selfmodifying_code:  Whether we aggressively support self-modifying code. When enabled, emulation
+    :param selfmodifying_code:          Whether we aggressively support self-modifying code. When enabled, emulation
                                         will try to read code from the current state instead of the original memory,
                                         regardless of the current memory protections.
-    :type support_selfmodifying_code:   bool
     :param store_function:              A function that defines how the Project should be stored. Default to pickling.
     :param load_function:               A function that defines how the Project should be loaded. Default to unpickling.
     :param analyses_preset:             The plugin preset for the analyses provider (i.e. Analyses instance).
@@ -82,7 +82,8 @@ class Project:
     :ivar analyses:     The available analyses.
     :type analyses:     angr.analysis.Analyses
     :ivar entry:        The program entrypoint.
-    :ivar factory:      Provides access to important analysis elements such as path groups and symbolic execution results.
+    :ivar factory:      Provides access to important analysis elements such as path groups and symbolic execution
+                        results.
     :type factory:      AngrObjectFactory
     :ivar filename:     The filename of the executable.
     :ivar loader:       The program loader.
@@ -90,33 +91,40 @@ class Project:
     :ivar storage:      Dictionary of things that should be loaded/stored with the Project.
     :type storage:      defaultdict(list)
     """
+
     analyses: "AnalysesHub"
     arch: archinfo.Arch
-    def __init__(self, thing,
-                 default_analysis_mode=None,
-                 ignore_functions=None,
-                 use_sim_procedures=True,
-                 exclude_sim_procedures_func=None,
-                 exclude_sim_procedures_list=(),
-                 arch=None, simos=None,
-                 engine=None,
-                 load_options: Dict[str, Any]=None,
-                 translation_cache=True,
-                 support_selfmodifying_code=False,
-                 store_function=None,
-                 load_function=None,
-                 analyses_preset=None,
-                 concrete_target=None,
-                 eager_ifunc_resolution=None,
-                 **kwargs):
 
+    def __init__(
+        self,
+        thing,
+        default_analysis_mode=None,
+        ignore_functions=None,
+        use_sim_procedures=True,
+        exclude_sim_procedures_func=None,
+        exclude_sim_procedures_list=(),
+        arch=None,
+        simos=None,
+        engine=None,
+        load_options: Dict[str, Any] = None,
+        translation_cache=True,
+        selfmodifying_code: bool = False,
+        support_selfmodifying_code: Optional[bool] = None,  # deprecated. use selfmodifying_code instead
+        store_function=None,
+        load_function=None,
+        analyses_preset=None,
+        concrete_target=None,
+        eager_ifunc_resolution=None,
+        **kwargs,
+    ):
         # Step 1: Load the binary
 
-        if load_options is None: load_options = {}
+        if load_options is None:
+            load_options = {}
 
         load_options.update(kwargs)
         if arch is not None:
-            load_options.update({'arch': arch})
+            load_options.update({"arch": arch})
         if isinstance(thing, cle.Loader):
             if load_options:
                 l.warning("You provided CLE options to angr but you also provided a completed cle.Loader object!")
@@ -125,7 +133,7 @@ class Project:
         elif isinstance(thing, cle.Backend):
             self.filename = thing.binary
             self.loader = cle.Loader(thing, **load_options)
-        elif hasattr(thing, 'read') and hasattr(thing, 'seek'):
+        elif hasattr(thing, "read") and hasattr(thing, "seek"):
             l.info("Loading binary from stream")
             self.filename = None
             self.loader = cle.Loader(thing, **load_options)
@@ -141,20 +149,22 @@ class Project:
         if isinstance(arch, str):
             self.arch = archinfo.arch_from_id(arch)  # may raise ArchError, let the user see this
         elif isinstance(arch, archinfo.Arch):
-            self.arch = arch # type: archinfo.Arch
+            self.arch: archinfo.Arch = arch
         elif arch is None:
             self.arch = self.loader.main_object.arch
         else:
             raise ValueError("Invalid arch specification.")
         # Step 3: Set some defaults and set the public and private properties
         if not default_analysis_mode:
-            default_analysis_mode = 'symbolic'
+            default_analysis_mode = "symbolic"
         if not ignore_functions:
             ignore_functions = []
 
         if isinstance(exclude_sim_procedures_func, types.LambdaType):
-            l.warning("Passing a lambda type as the exclude_sim_procedures_func argument to "
-                      "Project causes the resulting object to be un-serializable.")
+            l.warning(
+                "Passing a lambda type as the exclude_sim_procedures_func argument to "
+                "Project causes the resulting object to be un-serializable."
+            )
 
         self._sim_procedures = {}
 
@@ -162,13 +172,14 @@ class Project:
 
         # It doesn't make any sense to have auto_load_libs
         # if you have the concrete target, let's warn the user about this.
-        if self.concrete_target and load_options.get('auto_load_libs', None):
-
-            l.critical("Incompatible options selected for this project, please disable auto_load_libs if "
-                       "you want to use a concrete target.")
+        if self.concrete_target and load_options.get("auto_load_libs", None):
+            l.critical(
+                "Incompatible options selected for this project, please disable auto_load_libs if "
+                "you want to use a concrete target."
+            )
             raise Exception("Incompatible options for the project")
 
-        if self.concrete_target and self.arch.name not in ['X86', 'AMD64', 'ARMHF', 'MIPS32']:
+        if self.concrete_target and self.arch.name not in ["X86", "AMD64", "ARMHF", "MIPS32"]:
             l.critical("Concrete execution does not support yet the selected architecture. Aborting.")
             raise Exception("Incompatible options for the project")
 
@@ -177,12 +188,20 @@ class Project:
         self._exclude_sim_procedures_list = exclude_sim_procedures_list
         self.use_sim_procedures = use_sim_procedures
         self._ignore_functions = ignore_functions
-        self._support_selfmodifying_code = support_selfmodifying_code
+        self.selfmodifying_code = selfmodifying_code
         self._translation_cache = translation_cache
         self._eager_ifunc_resolution = eager_ifunc_resolution
-        self._executing = False # this is a flag for the convenience API, exec() and terminate_execution() below
+        self._executing = False  # this is a flag for the convenience API, exec() and terminate_execution() below
 
-        if self._support_selfmodifying_code:
+        # deprecation warning
+        if support_selfmodifying_code is not None:
+            l.warning(
+                'Parameter "support_selfmodifying_code" is deprecated. Please use "selfmodifying_code"'
+                ' instead. "support_selfmodifying_code" will be removed in the near future.'
+            )
+            self.selfmodifying_code = bool(support_selfmodifying_code)
+
+        if self.selfmodifying_code:
             if self._translation_cache is True:
                 self._translation_cache = False
                 l.warning("Disabling IRSB translation cache because support for self-modifying code is enabled.")
@@ -206,7 +225,7 @@ class Project:
 
         # Step 5: determine the guest OS
         if isinstance(simos, type) and issubclass(simos, SimOS):
-            self.simos = simos(self) #pylint:disable=invalid-name
+            self.simos = simos(self)  # pylint:disable=invalid-name
         elif isinstance(simos, str):
             self.simos = os_mapping[simos](self)
         elif simos is None:
@@ -236,7 +255,7 @@ class Project:
         Initializes self.analyses using a given preset.
         """
         self.analyses = AnalysesHub(self)
-        self.analyses.use_plugin_preset(self._analyses_preset if self._analyses_preset is not None else 'default')
+        self.analyses.use_plugin_preset(self._analyses_preset if self._analyses_preset is not None else "default")
 
     def _register_object(self, obj, sim_proc_arch):
         """
@@ -282,8 +301,12 @@ class Project:
                 # library names. The backend creates the Relocation objects, but leaves them unresolved so that
                 # we can try to guess them here.
                 if reloc.owner.guess_simprocs:
-                    l.debug("Looking for matching SimProcedure for unresolved %s from %s with hint %s",
-                            func.name, reloc.owner, reloc.owner.guess_simprocs_hint)
+                    l.debug(
+                        "Looking for matching SimProcedure for unresolved %s from %s with hint %s",
+                        func.name,
+                        reloc.owner,
+                        reloc.owner.guess_simprocs_hint,
+                    )
                     self._guess_simprocedure(func, reloc.owner.guess_simprocs_hint)
                 else:
                     l.debug("Ignoring unresolved import '%s' from %s ...?", func.name, reloc.owner)
@@ -297,8 +320,7 @@ class Project:
             # check if we actually can and want to replace it with a SimProcedure.
             # We opt out of this step if it is blacklisted by ignore_functions, which
             # will cause it to be replaced by ReturnUnconstrained later.
-            if export.owner is not self.loader._extern_object and \
-                    export.name not in self._ignore_functions:
+            if export.owner is not self.loader._extern_object and export.name not in self._ignore_functions:
                 if self._check_user_blacklists(export.name):
                     continue
                 owner_name = export.owner.provides
@@ -350,11 +372,14 @@ class Project:
                         the_lib = missing_libs[0]
                         if export.name and export.name.startswith("_Z"):
                             # GNU C++ name. Use a C++ library to create the stub
-                            if 'libstdc++.so' in SIM_LIBRARIES:
-                                the_lib = SIM_LIBRARIES['libstdc++.so']
+                            if "libstdc++.so" in SIM_LIBRARIES:
+                                the_lib = SIM_LIBRARIES["libstdc++.so"]
                             else:
-                                l.critical("Does not find any C++ library in SIM_LIBRARIES. We may not correctly "
-                                           "create the stub or resolve the function prototype for name %s.", export.name)
+                                l.critical(
+                                    "Does not find any C++ library in SIM_LIBRARIES. We may not correctly "
+                                    "create the stub or resolve the function prototype for name %s.",
+                                    export.name,
+                                )
 
                         self.hook_symbol(export.rebased_addr, the_lib.get(export.name, sim_proc_arch))
 
@@ -366,7 +391,10 @@ class Project:
             # use the vanilla ReturnUnconstrained, assuming that this isn't a weak func
             elif not func.is_weak:
                 l.info("Using stub SimProcedure for unresolved %s", export.name)
-                self.hook_symbol(export.rebased_addr, SIM_PROCEDURES['stubs']['ReturnUnconstrained'](display_name=export.name, is_stub=True))
+                self.hook_symbol(
+                    export.rebased_addr,
+                    SIM_PROCEDURES["stubs"]["ReturnUnconstrained"](display_name=export.name, is_stub=True),
+                )
 
     def _guess_simprocedure(self, f, hint):
         """
@@ -398,17 +426,17 @@ class Project:
         Has symbol name `f` been marked for exclusion by any of the user
         parameters?
         """
-        return not self.use_sim_procedures or \
-            f in self._exclude_sim_procedures_list or \
-            f in self._ignore_functions or \
-            (self._exclude_sim_procedures_func is not None and self._exclude_sim_procedures_func(f))
-
+        return (
+            not self.use_sim_procedures
+            or f in self._exclude_sim_procedures_list
+            or f in self._ignore_functions
+            or (self._exclude_sim_procedures_func is not None and self._exclude_sim_procedures_func(f))
+        )
 
     @staticmethod
     def _addr_to_str(addr):
         return "%s" % repr(addr) if isinstance(addr, SootAddressDescriptor) else "%#x" % addr
 
-
     #
     # Public methods
     # They're all related to hooking!
@@ -447,15 +475,18 @@ class Project:
             # if we haven't been passed a thing to hook with, assume we're being used as a decorator
             return self._hook_decorator(addr, length=length, kwargs=kwargs)
 
-        if kwargs is None: kwargs = {}
+        if kwargs is None:
+            kwargs = {}
 
-        l.debug('hooking %s with %s', self._addr_to_str(addr), str(hook))
+        l.debug("hooking %s with %s", self._addr_to_str(addr), str(hook))
 
         if self.is_hooked(addr):
             if replace is True:
                 pass
             elif replace is False:
-                l.warning("Address is already hooked, during hook(%s, %s). Not re-hooking.", self._addr_to_str(addr), hook)
+                l.warning(
+                    "Address is already hooked, during hook(%s, %s). Not re-hooking.", self._addr_to_str(addr), hook
+                )
                 return
             else:
                 l.warning("Address is already hooked, during hook(%s, %s). Re-hooking.", self._addr_to_str(addr), hook)
@@ -464,7 +495,7 @@ class Project:
             raise TypeError("Please instanciate your SimProcedure before hooking with it")
 
         if callable(hook):
-            hook = SIM_PROCEDURES['stubs']['UserHook'](user_func=hook, length=length, **kwargs)
+            hook = SIM_PROCEDURES["stubs"]["UserHook"](user_func=hook, length=length, **kwargs)
 
         self._sim_procedures[addr] = hook
 
@@ -554,6 +585,21 @@ class Project:
         self.hook(hook_addr, simproc, kwargs=kwargs, replace=replace)
         return hook_addr
 
+    def symbol_hooked_by(self, symbol_name) -> Optional[SimProcedure]:
+        """
+        Return the SimProcedure, if it exists, for the given symbol name.
+
+        :param str symbol_name: Name of the symbol.
+
+        :returns:    None if the address is not hooked.
+        """
+        sym = self.loader.find_symbol(symbol_name)
+        if sym is None:
+            l.warning("Could not find symbol %s", symbol_name)
+            return False
+        hook_addr, _ = self.simos.prepare_function_symbol(symbol_name, basic_addr=sym.rebased_addr)
+        return self.hooked_by(hook_addr)
+
     def is_symbol_hooked(self, symbol_name):
         """
         Check if a symbol is already hooked.
@@ -580,8 +626,10 @@ class Project:
             l.warning("Could not find symbol %s", symbol_name)
             return False
         if sym.owner is self.loader._extern_object:
-            l.warning("Refusing to unhook external symbol %s, replace it with another hook if you want to change it",
-                      symbol_name)
+            l.warning(
+                "Refusing to unhook external symbol %s, replace it with another hook if you want to change it",
+                symbol_name,
+            )
             return False
 
         hook_addr, _ = self.simos.prepare_function_symbol(symbol_name, basic_addr=sym.rebased_addr)
@@ -597,7 +645,6 @@ class Project:
         """
         new_sim_procedures = {}
         for key_address, simproc_obj in self._sim_procedures.items():
-
             # if we don't want stubs during the sync let's skip those, we will execute the real function.
             if not stubs_on_sync and simproc_obj.is_stub:
                 continue
@@ -675,7 +722,14 @@ class Project:
             self.store_function, self.load_function = None, None
             # ignore analyses. we re-initialize analyses when restoring from pickling so that we do not lose any newly
             # added analyses classes
-            d = dict((k, v) for k, v in self.__dict__.items() if k not in {'analyses', })
+            d = {
+                k: v
+                for k, v in self.__dict__.items()
+                if k
+                not in {
+                    "analyses",
+                }
+            }
             return d
         finally:
             self.store_function, self.load_function = store_func, load_func
@@ -686,30 +740,30 @@ class Project:
             self._initialize_analyses_hub()
         except AngrNoPluginError:
             l.warning("Plugin preset %s does not exist any more. Fall back to the default preset.")
-            self._analyses_preset = 'default'
+            self._analyses_preset = "default"
             self._initialize_analyses_hub()
 
     def _store(self, container):
         # If container is a filename.
         if isinstance(container, str):
-            with open(container, 'wb') as f:
+            with open(container, "wb") as f:
                 try:
                     pickle.dump(self, f, pickle.HIGHEST_PROTOCOL)
-                except RuntimeError as e: # maximum recursion depth can be reached here
+                except RuntimeError as e:  # maximum recursion depth can be reached here
                     l.error("Unable to store Project: '%s' during pickling", e)
 
         # If container is an open file.
         elif isinstance(container, IOBase):
             try:
                 pickle.dump(self, container, pickle.HIGHEST_PROTOCOL)
-            except RuntimeError as e: # maximum recursion depth can be reached here
+            except RuntimeError as e:  # maximum recursion depth can be reached here
                 l.error("Unable to store Project: '%s' during pickling", e)
 
         # If container is just a variable.
         else:
             try:
                 container = pickle.dumps(self, pickle.HIGHEST_PROTOCOL)
-            except RuntimeError as e: # maximum recursion depth can be reached here
+            except RuntimeError as e:  # maximum recursion depth can be reached here
                 l.error("Unable to store Project: '%s' during pickling", e)
 
     @staticmethod
@@ -717,7 +771,7 @@ class Project:
         if isinstance(container, str):
             # If container is a filename.
             if all(c in string.printable for c in container) and os.path.exists(container):
-                with open(container, 'rb') as f:
+                with open(container, "rb") as f:
                     return pickle.load(f)
 
             # If container is a pickle string.
@@ -734,14 +788,14 @@ class Project:
             return None
 
     def __repr__(self):
-        return '<Project %s>' % (self.filename if self.filename is not None else 'loaded from stream')
+        return "<Project %s>" % (self.filename if self.filename is not None else "loaded from stream")
 
     #
     # Compatibility
     #
 
     @property
-    @deprecated(replacement='simos')
+    @deprecated(replacement="simos")
     def _simos(self):
         return self.simos
 
