@@ -1,4 +1,3 @@
-
 import cle
 import io
 import logging
@@ -11,14 +10,18 @@ from ..errors import SimConcreteRegisterError
 from archinfo import ArchX86, ArchAMD64
 
 l = logging.getLogger("state_plugin.concrete")
-#l.setLevel(logging.DEBUG)
+# l.setLevel(logging.DEBUG)
 
 
 class Concrete(SimStatePlugin):
-    def __init__(self, segment_registers_initialized=False, segment_registers_callback_initialized=False,
-                 whitelist=None, fs_register_bp=None, already_sync_objects_addresses=None,
-                 ):
-
+    def __init__(
+        self,
+        segment_registers_initialized=False,
+        segment_registers_callback_initialized=False,
+        whitelist=None,
+        fs_register_bp=None,
+        already_sync_objects_addresses=None,
+    ):
         super().__init__()
 
         self.segment_registers_initialized = segment_registers_initialized
@@ -40,12 +43,13 @@ class Concrete(SimStatePlugin):
             self.already_sync_objects_addresses = already_sync_objects_addresses
 
     def copy(self, _memo):
-        conc = Concrete(segment_registers_initialized=self.segment_registers_initialized,
-                        segment_registers_callback_initialized=self.segment_registers_callback_initialized,
-                        whitelist=list(self.whitelist),
-                        fs_register_bp=self.fs_register_bp,
-                        already_sync_objects_addresses=list(self.already_sync_objects_addresses)
-                        )
+        conc = Concrete(
+            segment_registers_initialized=self.segment_registers_initialized,
+            segment_registers_callback_initialized=self.segment_registers_callback_initialized,
+            whitelist=list(self.whitelist),
+            fs_register_bp=self.fs_register_bp,
+            already_sync_objects_addresses=list(self.already_sync_objects_addresses),
+        )
         return conc
 
     def merge(self, _others, _merge_conditions, _common_ancestor=None):
@@ -83,7 +87,7 @@ class Concrete(SimStatePlugin):
                 gdt = state.project.simos.initialize_gdt_x86(state, concr_target)
                 state.concrete.whitelist.append((gdt.addr, gdt.addr + gdt.limit))
 
-            state.inspect.remove_breakpoint('reg_read', bp=state.concrete.fs_register_bp)
+            state.inspect.remove_breakpoint("reg_read", bp=state.concrete.fs_register_bp)
             state.concrete.segment_registers_initialized = True
 
             state.concrete.fs_register_bp = None
@@ -105,7 +109,6 @@ class Concrete(SimStatePlugin):
         to_sync_register = list(filter(lambda x: x.concrete, self.state.arch.register_list))
 
         for register in to_sync_register:
-
             # before let's sync all the subregisters of the current register.
             # sometimes this can be helpful ( i.e. ymmm0 e xmm0 )
             if register.subregisters:
@@ -128,17 +131,20 @@ class Concrete(SimStatePlugin):
         # flush the angr memory in order to synchronize them with the content of the
         # concrete process memory when a read/write to the page is performed
         self.state.memory.flush_pages(self.whitelist)
-        l.info("Exiting SimEngineConcrete: simulated address %x concrete address %x ", self.state.addr,
-               target.read_register("pc"))
+        l.info(
+            "Exiting SimEngineConcrete: simulated address %x concrete address %x ",
+            self.state.addr,
+            target.read_register("pc"),
+        )
 
         # now we have to register a SimInspect in order to synchronize the segments register
         # on demand when the symbolic execution accesses it
-        if self.state.project.arch.name in ['X86', 'AMD64'] and not self.segment_registers_callback_initialized:
+        if self.state.project.arch.name in ["X86", "AMD64"] and not self.segment_registers_callback_initialized:
             segment_register_name = self.state.project.simos.get_segment_register_name()
             if segment_register_name:
-                self.fs_register_bp = self.state.inspect.b('reg_read',
-                                                           reg_read_offset=segment_register_name,
-                                                           action=_sync_segments)
+                self.fs_register_bp = self.state.inspect.b(
+                    "reg_read", reg_read_offset=segment_register_name, action=_sync_segments
+                )
 
                 self.segment_registers_callback_initialized = True
 
@@ -151,15 +157,19 @@ class Concrete(SimStatePlugin):
             try:
                 reg_value = target.read_register(register_name)
                 setattr(self.state.regs, register_name, reg_value)
-                l.debug("Register: %s value: %x ", register_name, self.state.solver.eval(getattr(self.state.regs,
-                                                                                                 register_name),
-                                                                                         cast_to=int))
+                l.debug(
+                    "Register: %s value: %x ",
+                    register_name,
+                    self.state.solver.eval(getattr(self.state.regs, register_name), cast_to=int),
+                )
             except SimConcreteRegisterError as exc:
-                l.debug("Can't set register %s reason: %s, if this register is not used "
-                        "this message can be ignored", register_name, exc)
+                l.debug(
+                    "Can't set register %s reason: %s, if this register is not used this message can be ignored",
+                    register_name,
+                    exc,
+                )
 
     def _sync_cle(self, target):
-
         def _check_mapping_name(cle_mapping_name, concrete_mapping_name):
             if cle_mapping_name == concrete_mapping_name:
                 return True
@@ -197,16 +207,21 @@ class Concrete(SimStatePlugin):
                     if self.state.project.loader.main_object.check_magic_compatibility(io.BytesIO(result)):
                         if mapped_object.mapped_base == mmap.start_address:
                             # We already have the correct address for this memory mapping
-                            l.debug("Object %s is already rebased correctly at 0x%x", binary_name,
-                                    mapped_object.mapped_base)
+                            l.debug(
+                                "Object %s is already rebased correctly at 0x%x", binary_name, mapped_object.mapped_base
+                            )
                             self.already_sync_objects_addresses.append(mmap.name)
 
                             break  # object has been synchronized, move to the next one!
 
                         # rebase the object if the CLE address doesn't match the real one,
                         # this can happen with PIE binaries and libraries.
-                        l.debug("Remapping object %s mapped at address 0x%x at address 0x%x", binary_name,
-                                mapped_object.mapped_base, mmap.start_address)
+                        l.debug(
+                            "Remapping object %s mapped at address 0x%x at address 0x%x",
+                            binary_name,
+                            mapped_object.mapped_base,
+                            mmap.start_address,
+                        )
 
                         old_mapped_base = mapped_object.mapped_base
                         mapped_object.mapped_base = mmap.start_address  # Rebase now!
@@ -219,7 +234,6 @@ class Concrete(SimStatePlugin):
                         break  # object has been synchronized, move to the next one!
 
     def _sync_simproc(self):
-
         l.debug("Restoring SimProc using concrete memory")
 
         for reloc in self.state.project.loader.main_object.relocs:
@@ -227,10 +241,12 @@ class Concrete(SimStatePlugin):
                 l.debug("Trying to re-hook SimProc %s", reloc.symbol.name)
                 # l.debug("reloc.rebased_addr: %#x " % reloc.rebased_addr)
 
-                if self.state.project.simos.name == 'Win32':
-                    func_address = self.state.project.concrete_target.read_memory(reloc.rebased_addr, self.state.arch.bytes)
+                if self.state.project.simos.name == "Win32":
+                    func_address = self.state.project.concrete_target.read_memory(
+                        reloc.rebased_addr, self.state.arch.bytes
+                    )
                     func_address = struct.unpack(self.state.project.arch.struct_fmt(), func_address)[0]
-                elif self.state.project.simos.name == 'Linux':
+                elif self.state.project.simos.name == "Linux":
                     try:
                         func_address = self.state.project.loader.main_object.plt[reloc.symbol.name]
                     except KeyError:
@@ -251,16 +267,25 @@ class Concrete(SimStatePlugin):
                         # calculating the new real address
                         new_relative_address = func_address - owner_obj.mapped_base
 
-                        new_func_symbol = cle.backends.Symbol(owner_obj, old_func_symbol.name, new_relative_address,
-                                                              old_func_symbol.size, old_func_symbol.type)
+                        new_func_symbol = cle.backends.Symbol(
+                            owner_obj,
+                            old_func_symbol.name,
+                            new_relative_address,
+                            old_func_symbol.size,
+                            old_func_symbol.type,
+                        )
 
                         for new_reloc in self.state.project.loader.find_relevant_relocations(old_func_symbol.name):
-                            if new_reloc.symbol.name == new_func_symbol.name and \
-                                    new_reloc.value != new_func_symbol.rebased_addr:
-                                l.debug("Updating CLE symbols metadata, moving %s from 0x%x to 0x%x",
-                                        new_reloc.symbol.name,
-                                        new_reloc.value,
-                                        new_func_symbol.rebased_addr)
+                            if (
+                                new_reloc.symbol.name == new_func_symbol.name
+                                and new_reloc.value != new_func_symbol.rebased_addr
+                            ):
+                                l.debug(
+                                    "Updating CLE symbols metadata, moving %s from 0x%x to 0x%x",
+                                    new_reloc.symbol.name,
+                                    new_reloc.value,
+                                    new_func_symbol.rebased_addr,
+                                )
 
                                 new_reloc.resolve(new_func_symbol)
                                 new_reloc.relocate([])
@@ -268,4 +293,5 @@ class Concrete(SimStatePlugin):
 
 from ..sim_state import SimState
 from .. import sim_options as options
-SimState.register_default('concrete', Concrete)
+
+SimState.register_default("concrete", Concrete)
