@@ -1,4 +1,4 @@
-
+from typing import Tuple, Optional, Dict, List
 from collections import defaultdict
 import logging
 
@@ -25,6 +25,65 @@ def shallow_reverse(g) -> networkx.DiGraph:
     return new_g
 
 
+def inverted_idoms(graph: networkx.DiGraph) -> Tuple[networkx.DiGraph, Optional[Dict]]:
+    """
+    Invert the given graph and generate the immediate dominator tree on the inverted graph. This is useful for
+    computing post-dominators.
+
+    :param graph:   The graph to invert and generate immediate dominator tree for.
+    :return:        A tuple of the inverted graph and the immediate dominator tree.
+    """
+
+    end_nodes = {n for n in graph.nodes() if graph.out_degree(n) == 0}
+    inverted_graph: networkx.DiGraph = shallow_reverse(graph)
+    if end_nodes:
+        if len(end_nodes) > 1:
+            # make sure there is only one end node
+            dummy_node = "DUMMY_NODE"
+            for end_node in end_nodes:
+                inverted_graph.add_edge(dummy_node, end_node)
+            endnode = dummy_node
+        else:
+            endnode = next(iter(end_nodes))  # pick the end node
+
+        idoms = networkx.immediate_dominators(inverted_graph, endnode)
+    else:
+        idoms = None
+    return inverted_graph, idoms
+
+
+def to_acyclic_graph(
+    graph: networkx.DiGraph, ordered_nodes: Optional[List] = None, loop_heads: Optional[List] = None
+) -> networkx.DiGraph:
+    """
+    Convert a given DiGraph into an acyclic graph.
+
+    :param graph:           The graph to convert.
+    :param ordered_nodes:   A list of nodes sorted in a topological order.
+    :param loop_heads:      A list of known loop head nodes.
+    :return:                The converted acyclic graph.
+    """
+
+    if ordered_nodes is None:
+        # take the quasi-topological order of the graph
+        from angr.analyses.cfg.cfg_utils import CFGUtils  # pylint:disable=import-outside-toplevel
+
+        ordered_nodes = CFGUtils.quasi_topological_sort_nodes(graph, loop_heads=loop_heads)
+
+    acyclic_graph = networkx.DiGraph()
+
+    # add each node and its edge into the graph
+    visited = set()
+    for node in ordered_nodes:
+        visited.add(node)
+        acyclic_graph.add_node(node)
+        for successor in graph.successors(node):
+            if successor not in visited:
+                acyclic_graph.add_edge(node, successor)
+
+    return acyclic_graph
+
+
 def dfs_back_edges(graph, start_node):
     """
     Do a DFS traversal of the graph, and return with the back edges.
@@ -48,12 +107,10 @@ def dfs_back_edges(graph, start_node):
                 if child in visited:
                     yield node, child
                 else:
-                    for s, t in _dfs_back_edges_core(child):
-                        yield s, t
+                    yield from _dfs_back_edges_core(child)
         finished.add(node)
 
-    for s,t in _dfs_back_edges_core(start_node):
-        yield s,t
+    yield from _dfs_back_edges_core(start_node)
 
 
 def subgraph_between_nodes(graph, source, frontier, include_frontier=False):
@@ -79,7 +136,7 @@ def subgraph_between_nodes(graph, source, frontier, include_frontier=False):
         raise KeyError("Source node or frontier nodes are not in the source graph.")
 
     # BFS on graph and add new nodes to g0
-    queue = [ source ]
+    queue = [source]
     traversed = set()
 
     frontier = set(frontier)
@@ -98,10 +155,18 @@ def subgraph_between_nodes(graph, source, frontier, include_frontier=False):
                     break
 
     # recursively remove all nodes that have less than two neighbors
-    to_remove = [ n for n in g0.nodes() if n not in frontier and n is not source and (g0.out_degree[n] == 0 or g0.in_degree[n] == 0) ]
+    to_remove = [
+        n
+        for n in g0.nodes()
+        if n not in frontier and n is not source and (g0.out_degree[n] == 0 or g0.in_degree[n] == 0)
+    ]
     while to_remove:
         g0.remove_nodes_from(to_remove)
-        to_remove = [ n for n in g0.nodes() if n not in frontier and n is not source and (g0.out_degree[n] == 0 or g0.in_degree[n] == 0) ]
+        to_remove = [
+            n
+            for n in g0.nodes()
+            if n not in frontier and n is not source and (g0.out_degree[n] == 0 or g0.in_degree[n] == 0)
+        ]
 
     if not include_frontier:
         # remove the frontier nodes
@@ -121,10 +186,12 @@ def dominates(idom, dominator_node, node):
             n = None
     return False
 
+
 #
 # Dominance frontier
 #
 
+
 def compute_dominance_frontier(graph, domtree):
     """
     Compute a dominance frontier based on the given post-dominator tree.
@@ -141,7 +208,6 @@ def compute_dominance_frontier(graph, domtree):
 
     # Perform a post-order search on the dominator tree
     for x in networkx.dfs_postorder_nodes(domtree):
-
         if x not in graph:
             # Skip nodes that are not in the graph
             continue
@@ -181,13 +247,13 @@ class TemporaryNode:
     Used as the start node and end node in post-dominator tree generation. Also used in some test cases.
     """
 
-    __slots__ = ['_label']
+    __slots__ = ["_label"]
 
     def __init__(self, label):
         self._label = label
 
     def __repr__(self):
-        return 'TN[%s]' % self._label
+        return "TN[%s]" % self._label
 
     def __eq__(self, other):
         if isinstance(other, TemporaryNode) and other._label == self._label:
@@ -195,7 +261,7 @@ class TemporaryNode:
         return False
 
     def __hash__(self):
-        return hash(('TemporaryNode', self._label))
+        return hash(("TemporaryNode", self._label))
 
 
 class ContainerNode:
@@ -206,7 +272,7 @@ class ContainerNode:
     original object.
     """
 
-    __slots__ = ['_obj', 'index']
+    __slots__ = ["_obj", "index"]
 
     def __init__(self, obj):
         self._obj = obj
@@ -222,7 +288,7 @@ class ContainerNode:
         return False
 
     def __hash__(self):
-        return hash(('CN', self._obj))
+        return hash(("CN", self._obj))
 
     def __repr__(self):
         return "CN[%s]" % repr(self._obj)
@@ -232,7 +298,6 @@ class Dominators:
     dom: networkx.DiGraph
 
     def __init__(self, graph, entry_node, successors_func=None, reverse=False):
-
         self._l = logging.getLogger("utils.graph.dominators")
         self._graph_successors_func = successors_func
 
@@ -330,7 +395,6 @@ class Dominators:
         self.prepared_graph = _prepared_graph
 
     def _prepare_graph(self, graph, entry):
-
         # We want to reverse the graph, and label each node according to its order in a DFS
         new_graph = networkx.DiGraph()
 
@@ -424,8 +488,9 @@ class Dominators:
             if counter >= all_nodes_count:
                 break
 
-            self._l.debug("%d nodes are left out during the DFS. They must formed a cycle themselves.",
-                          all_nodes_count - counter)
+            self._l.debug(
+                "%d nodes are left out during the DFS. They must formed a cycle themselves.", all_nodes_count - counter
+            )
             # Find those nodes
             leftovers = [s for s in traversed_nodes if s not in scanned_nodes]
             new_graph.add_edge(start_node, leftovers[0])
@@ -453,8 +518,10 @@ class Dominators:
     def _pd_compress(self, v):
         if self._ancestor[self._ancestor[v.index].index] is not None:
             self._pd_compress(self._ancestor[v.index])
-            if self._semi[self._label[self._ancestor[v.index].index].index].index < \
-                    self._semi[self._label[v.index].index].index:
+            if (
+                self._semi[self._label[self._ancestor[v.index].index].index].index
+                < self._semi[self._label[v.index].index].index
+            ):
                 self._label[v.index] = self._label[self._ancestor[v.index].index]
             self._ancestor[v.index] = self._ancestor[self._ancestor[v.index].index]
 
