@@ -1,5 +1,4 @@
 # pylint:disable=no-member
-import sys
 import time
 from functools import wraps
 from collections import defaultdict
@@ -15,21 +14,20 @@ time_distribution = defaultdict(list)
 def timethis(func):
     @wraps(func)
     def timed_func(*args, **kwargs):
-
         if TIMING:
-            if sys.version_info >= (3,7):
-                _t = lambda: time.perf_counter_ns() / 1000000
-            else:
-                _t = lambda: time.time() * 1000
+
+            def _t():
+                return time.perf_counter_ns() / 1000000
+
             start = _t()
             r = func(*args, **kwargs)
             millisec = _t() - start
             sec = millisec / 1000
             if PRINT:
                 if sec > 1.0:
-                    print("[timing] %s took %f seconds (%f milliseconds)." % (func.__name__, sec, millisec))
+                    print(f"[timing] {func.__name__} took {sec:f} seconds ({millisec:f} milliseconds).")
                 else:
-                    print("[timing] %s took %f milliseconds." % (func.__name__, millisec))
+                    print(f"[timing] {func.__name__} took {millisec:f} milliseconds.")
             total_time[func] += millisec
             if TIME_DISTRIBUTION:
                 time_distribution[func].append(millisec)
