@@ -132,6 +132,7 @@ namespace std {
 struct memory_value_t {
 	uint64_t address;
 	uint8_t value;
+	bool is_value_set;
 	bool is_value_symbolic;
 
 	bool operator==(const memory_value_t &other_mem_value) const {
@@ -145,16 +146,19 @@ struct memory_value_t {
 	memory_value_t() {
 		address = 0;
 		value = 0;
+		is_value_set = false;
 		is_value_symbolic = false;
 	}
 };
 
 struct mem_read_result_t {
+	address_t first_byte_addr;
 	std::vector<memory_value_t> memory_values;
 	bool is_mem_read_symbolic;
 	uint32_t read_size;
 
 	mem_read_result_t() {
+		first_byte_addr = 0;
 		memory_values.clear();
 		is_mem_read_symbolic = false;
 		read_size = 0;
@@ -367,6 +371,9 @@ struct vex_stmt_taint_entry_t {
 	// List of taint entities in ITE expression's condition, if any
 	std::unordered_set<taint_entity_t> ite_cond_entity_list;
 
+	// Indicates if statement has a floating point operation for which taint should not be propagated
+	bool floating_point_op_skip;
+
 	// Count number of bytes read from memory by the instruction
 	uint32_t mem_read_size;
 
@@ -382,7 +389,7 @@ struct vex_stmt_taint_entry_t {
 		return (sink == other_stmt.sink) && (sources == other_stmt.sources) &&
 			   (ite_cond_entity_list == other_stmt.ite_cond_entity_list) &&
 			   (has_memory_read == other_stmt.has_memory_read) &&
-			   (is_exit == other_stmt.is_exit) &&
+			   (is_exit == other_stmt.is_exit) && (floating_point_op_skip == other_stmt.floating_point_op_skip) &&
 			   (mem_read_size == other_stmt.mem_read_size) &&
 			   (mem_write_size == other_stmt.mem_write_size);
 	}
@@ -395,6 +402,7 @@ struct vex_stmt_taint_entry_t {
 		sink.reset();
 		sources.clear();
 		ite_cond_entity_list.clear();
+		floating_point_op_skip = false;
 		has_memory_read = false;
 		is_exit = false;
 		mem_read_size = 0;
@@ -439,6 +447,7 @@ struct stop_details_t {
 struct processed_vex_expr_t {
 	std::unordered_set<taint_entity_t> taint_sources;
 	std::unordered_set<taint_entity_t> ite_cond_entities;
+	bool has_floating_point_op_to_skip;
 	bool has_unsupported_expr;
 	stop_t unsupported_expr_stop_reason;
 	uint32_t mem_read_size;
@@ -451,6 +460,7 @@ struct processed_vex_expr_t {
 	void reset() {
 		taint_sources.clear();
 		ite_cond_entities.clear();
+		has_floating_point_op_to_skip = false;
 		has_unsupported_expr = false;
 		mem_read_size = 0;
 		value_size = -1;
@@ -549,9 +559,6 @@ class State {
 	// List of symbolic statements in processed basic blocks that need not be re-executed. Will be removed on commit.
 	std::unordered_map<uint32_t, std::unordered_set<uint32_t>> symbolic_stmts_to_erase;
 
-	// List of register values at start of block
-	std::map<vex_reg_offset_t, register_value_t> block_start_reg_values;
-
 	// Similar to memory reads in a block, we track the state of registers and VEX temps when
 	// propagating taint in a block for easy rollback if we need to abort due to read from/write to
 	// a symbolic address
@@ -611,13 +618,22 @@ class State {
 	// Symbolic memory dependencies of all symbolic VEX statements in current block.
 	std::unordered_set<address_t> block_symbolic_mem_deps;
 
+	// List of all values read from memory in current block
+	std::vector<memory_value_t> block_mem_reads_data;
+
+	// Result of all memory reads executed. VEX statement ID -> memory read result
+	std::unordered_map<int64_t, mem_read_result_t> block_mem_reads_map;
+
+	// List of memory read addresses and VEX statements that read values from them
+	std::unordered_map<address_t, std::unordered_set<int64_t>> block_mem_read_addr_details;
+
 	// Private functions
 
 	std::pair<taint_t *, uint8_t *> page_lookup(address_t address) const;
 
 	void compute_slice_of_stmt(vex_stmt_details_t &instr);
 	vex_stmt_details_t compute_vex_stmt_details(const vex_stmt_taint_entry_t &vex_stmt_taint_entry);
-	void get_register_value(uint64_t vex_reg_offset, uint8_t *out_reg_value) const;
+	void get_register_value(uint64_t vex_reg_offset, uint8_t *out_reg_value, bool from_saved_context=false) const;
 	// Return list of all dependent instructions including dependencies of those dependent instructions
 	std::set<vex_stmt_details_t> get_list_of_dep_stmts(const vex_stmt_details_t &instr) const;
 
@@ -658,6 +674,9 @@ class State {
 	// Save values of concrete memory reads performed by an instruction and it's dependencies
 	void save_concrete_memory_deps(vex_stmt_details_t &instr);
 
+	// Save memory values of concrete deps
+	void save_mem_values(mem_read_result_t &mem_read_result);
+
 	// Inline functions
 
 	inline bool is_valid_dependency_register(vex_reg_offset_t reg_offset) const {
@@ -753,7 +772,7 @@ class State {
 		RegisterSet symbolic_registers; // tracking of symbolic registers
 		RegisterSet blacklisted_registers;  // Registers which shouldn't be saved as a concrete dependency
 		// Mapping of VEX offsets to unicorn register IDs and register sizes
-		std::unordered_map<vex_reg_offset_t, std::pair<unicorn_reg_id_t, uint64_t>> vex_to_unicorn_map;
+		std::map<vex_reg_offset_t, std::pair<unicorn_reg_id_t, uint64_t>> vex_to_unicorn_map;
 		// VEX CC registers
 		std::unordered_map<vex_reg_offset_t, uint64_t> vex_cc_regs;
 		RegisterSet artificial_vex_registers; // Artificial VEX registers
@@ -761,12 +780,6 @@ class State {
 		std::unordered_map<vex_reg_offset_t, std::pair<uint64_t, uint64_t>> cpu_flags;
 		stop_details_t stop_details;
 
-		// List of all values read from memory in current block
-		std::vector<memory_value_t> block_mem_reads_data;
-
-		// Result of all memory reads executed. VEX statement ID -> memory read result
-		std::unordered_map<int64_t, mem_read_result_t> block_mem_reads_map;
-
 		// Address of all bytes to which symbolic value is written in this run and number of writes to them
 		std::unordered_map<address_t, uint64_t> symbolic_mem_writes;
 
@@ -788,6 +801,12 @@ class State {
 
 		uc_cb_eventmem_t py_mem_callback;
 
+		// List of floating point operations
+		std::unordered_set<uint64_t> fp_ops_to_avoid;
+
+		// Floating point registers
+		std::pair<uint64_t, uint64_t> fp_reg_vex_data;
+
 		State(uc_engine *_uc, uint64_t cache_key, simos_t curr_os, bool symb_addrs, bool symb_cond, bool symb_syscalls);
 
 		~State() {
@@ -814,14 +833,14 @@ class State {
 
 		uc_err start(address_t pc, uint64_t step = 1);
 
-		void stop(stop_t reason, bool do_commit=false);
+		void stop(stop_t reason, bool do_commit=false, uint64_t commit_addr=0);
 
 		void step(address_t current_address, int32_t size, bool check_stop_points=true);
 
 		/*
 		* commit all memory actions.
 		*/
-		void commit();
+		void commit(uint64_t address);
 
 		/*
 		 * undo recent memory actions.
